***********************************************************************************************;
* DRAFT - CONFIDENTIAL		           									   		   			   ;
* Program:	Net EAS Revenues Model		           									   		   ;
* Date:		September 2020                                                     				   ;
* Author:	Analysis Group, Inc      														   ;
* Client: 	New York Independent System Operator											   ;
* Purpose:	Calculate 3-Year Average Net EAS Revenues for Peaking Unit						   ; 
***********************************************************************************************;

%let filepath = /* Enter filepath */;
%let output_name = EAS Model Output CT Dual Fuel;
/*Set to 'yes' for gas only*/
%let enable_gas_only = no;
%let enable_LOE = yes;

/*Default Modeling Period is September 2016 Through August 2019.*/

options mlogic mprint;
libname neteas "&filepath.\Datasets";
%let outpath = &filepath.\Output;
%let inpath = &filepath.\Input;

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 1. Load Annually Updated Parameters								   			   			   ;
* 2. Load DCR Set Parameters								   			   				   	   ;
* 3. Merge Together Price Data for Each Hour								   			   	   ;
* 4. Calculate Base DAM and RTD Energy Profits in Each Hour					   			   	   ;
* 5. DAM Energy Blocks - Set Day-Ahead Behavior and Determine if Real-Time Buyout Profitable   ;
* 6. DAM Non-Energy Hours - Determine if Units Commit Reserves or No Commitment 		       ;
* 7. RTD Energy Blocks - Determine if Profitable to Change Day-Ahead Non-Energy to 			   ;	
*						 Real-Time Energy Dispatch											   ;
* 8. RTD Non-Energy Hours - Determine if Units Buyout or Dispatch Real-Time Reserves 	  	   ;
* 9. EFORd Adjustment  	  	   												     			   ;
* 10. Run-Time Limit for NSPS 	  	   												   		   ;
* 11. Output Model Results																	   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 1. Load Annually Updated Parameters								   			   			   ;
* 	A: Energy LBMP																			   ;	
* 	B: Nonspin Reserve Prices																   ;
* 	C: Fuel Prices (Gas and Oil)															   ;
* 	D: Emission Prices (CO2, NOX, SO2)														   ;
* 	E: RS1																					   ;
* 	F: Modeling Dates																		   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*A: Energy LBMP*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = lbmp_dam 
	DBMS = XLSX replace;
	sheet = "DA LBMP - All Zones";
run;
data in_lbmp_dam (	drop = Eastern_Date_Hour Zone_PTID DAM_Zonal_Losses--DAM_Zonal_Price_Version
							rename = (DAM_Zonal_LBMP = DAM_LBMP Zone_Name = Zone)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set lbmp_dam;
	if Zone_Name = "N.Y.C." then Zone_Name = "NYC";
	else if Zone_Name = "HUD VL" then Zone_Name = "HUD_VL";
	DATE = datepart(Eastern_Date_Hour);
	TIME = timepart(Eastern_Date_Hour);
run;

/*RTD LMBP*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = lbmp_rtd
	DBMS = XLSX replace;
	sheet = "RT LBMP - All Zones";
run;
data in_lbmp_rtd (	drop = Eastern_Date_Hour Zone_PTID TWI_Zonal_Losses--TWI_Zonal_Price_Version
							rename = (TWI_Zonal_LBMP = RTD_LBMP Zone_Name = Zone)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set lbmp_rtd;
	if Zone_Name = "N.Y.C." then Zone_Name = "NYC";
	else if Zone_Name = "HUD VL" then Zone_Name = "HUD_VL";
	DATE = datepart(Eastern_Date_Hour);
	TIME = timepart(Eastern_Date_Hour);
run;

/*B: Nonspin Reserve Prices*/
/*DAM*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = anc_dam
	DBMS = XLSX replace;
	sheet = "DA Ancillary - All Zones";
run;
data in_anc_dam (	drop = Eastern_Date_Hour DAM_10_Min_Sync DAM_NYCA_Regulation_Capacity DAM_AS_Price_Version
							rename = (DAM_10_Min_Non_Sync = DAM_10MIN_NONSPIN 
									  DAM_30_Min_Non_Sync = DAM_30MIN
									  Zone_Name = Zone)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set anc_dam;
	if Zone_Name = "N.Y.C." then Zone_Name = "NYC";
	else if Zone_Name = "HUD VL" then Zone_Name = "HUD_VL";
	DATE = datepart(Eastern_Date_Hour);
	TIME = timepart(Eastern_Date_Hour);
run;

/*RTD*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = anc_rtd 
	DBMS = XLSX replace;
	sheet = "RT Ancillary - All Zones";
run;
data in_anc_rtd (	drop = 	Hour_Begin_Time_Stamp 
									Time_Zone
									Zone_PTID
									_10_Min_Spinning_Reserve 
									NYCA_Regulation_Capacity 
									NYCA_Regulation_Movement________
							rename = (	_10_Min_Non__Synchronous_Reserve = RTD_10MIN_NONSPIN
										_30_Min_Operating_Reserve = RTD_30MIN)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set anc_rtd;
	if Zone = "N.Y.C." then Zone = "NYC";
	else if Zone = "HUD VL" then Zone = "HUD_VL";
	DATE = datepart(Hour_Begin_Time_Stamp);
	TIME = timepart(Hour_Begin_Time_Stamp);
run;

/*Make adjustment for Daylight Savings Hour*/
%macro adjust_for_dls(data=);
	/*1. Split into duplicated (daylight savings) and non-duplicated hours*/
	proc sort nouniquekey data = &data. out = &data._dls uniqueout = &data._no_dls;
	      by DATE TIME ZONE;
	run;

	/*2. Recombine and increment 2nd daylight savings hour by 1 minute to make the timestamp unique. */
	/*	Model will process both records as distinct and full hours. */
	data &data._adj;
		set &data._dls (in = a) &data._no_dls (in = b);
		if a and mod(_N_,2) = 0 then TIME = TIME + '00:00:01't;
		proc sort; by DATE TIME ZONE;
	run;
%mend;
%adjust_for_dls(data = in_lbmp_dam);
%adjust_for_dls(data = in_lbmp_rtd);
%adjust_for_dls(data = in_anc_dam);
%adjust_for_dls(data = in_anc_rtd);

/*C: Fuel Prices*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_f_gas (where = (~missing(DATE)));
	sheet = "SNL Gas";
run;
data in_f_gas_lag;
	set in_f_gas;
	/*DATE = DATE + 1;*/ /* 9.18.2020 Note: statement removed per new gas price methodology */
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_f_oil (where = (~missing(DAY)));
	sheet = "EIA Oil";
	proc sort nodupkey; by DAY;
run;

/*D: Emission Prices*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_co2 (rename = (Auction_Clearing_Price = CO2_PRICE) where = (~missing(AUCTION_DATE)));
	sheet = "RGGI CO2";
	proc sort nodupkey; by AUCTION_DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_nox (rename = (Seasonal_NOX = NOX_SEASONAL_PRICE Annual_NOX = NOX_ANNUAL_PRICE) where = (~missing(DATE)));
	sheet = "SNL NOx";
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_so2_CSAPR  (rename = (ALLOWANCE_PRICE = SO2_PRICE) where = (~missing(DATE)));
	sheet = "SNL CSAPR SO2";
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_so2_arp  (rename = (ALLOWANCE_PRICE = SO2_PRICE) where = (~missing(DATE)));
	sheet = "SNL ARP SO2";
	proc sort nodupkey; by DATE;
run;
/*Use EPA auction data prior to SNL coverage (Jan 1, 2015)*/
data in_e_so2_arp_epa;
	DATE = 2005; SO2_PRICE = 702.51; output;
	DATE = 2006; SO2_PRICE = 883.1; output;
	DATE = 2007; SO2_PRICE = 444.39; output;
	DATE = 2008; SO2_PRICE = 389.91; output;
	DATE = 2009; SO2_PRICE = 69.74; output;
	DATE = 2010; SO2_PRICE = 37.71; output;
	DATE = 2011; SO2_PRICE = 2.81; output;
	DATE = 2012; SO2_PRICE = 0.67; output;
	DATE = 2013; SO2_PRICE = 0.28; output;
	DATE = 2014; SO2_PRICE = 0.45; output;
run;

/*E: RS1*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_rs1 (where = (~missing(YEAR)));
	sheet = "NYISO RS1";
	proc sort nodupkey; by YEAR;
run;

/*F: Modeling Dates*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_dates (where = (~missing(DATE)));
	sheet = "Dates";
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 2. Load DCR Set Parameters								   			   				   	   ;
/*	A: Fuel Transportation and Intraday*/
/*	B: Unit Characteristics */
/*  C: Modeling Decisions*/
/*  D: LOE Adjustment */
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*A: Fuel Transportation and Intraday*/
/*Natural Gas Transportation Costs (See IMM Table A-2)*/
	%let ros_fuel_trans = 0.27;
	%let hv_fuel_trans = 0.27;
	%let nyc_fuel_trans = 0.20;
	%let li_fuel_trans = 0.25;

/*Natural Gas Intraday Premium (See IMM Table A-2)*/
	%let ros_fuel_prem = 1.1;
	%let hv_fuel_prem = 1.1;
	%let nyc_fuel_prem = 1.2;
	%let li_fuel_prem = 1.3;
	%let ros_fuel_disc = 0.9;
	%let hv_fuel_disc = 0.9;
	%let nyc_fuel_disc = 0.8;
	%let li_fuel_disc = 0.7;

/*Gas excise taxes (See IMM Page A-23)*/
	%let nyc_gas_tax = 1.069;
	%let li_gas_tax = 1.01;
	
/*NYC 4.5% sales tax*/
	%let nyc_sales_tax = 1.045;

/*ULSD Transportation Costs (See IMM Table A-2)*/
	%let ros_ulsd_trans = 2;
	%let hv_ulsd_trans = 1.5;
	%let nyc_ulsd_trans = 1.5;
	%let li_ulsd_trans = 1.5;

/*B: Unit Characteristics (Provided by Lummus)*/
	%let ns10min_lms = "Yes";
	%let ns10min_f = "No";
	%let ns10min_w = "Yes";
	%let ns10min_wg = "Yes";
	%let ns10min_h = "No";
	/*Run-time Limits*/
	%let rt_limit_lms = 0.424;
	%let rt_limit_f = 0.384;
	%let rt_limit_w = 1.000;
	%let rt_limit_wg = 1.000;
	%let rt_limit_h = 0.409;

	/*Historical Average VSS revenues from NYISO data*/
	%let vss_revenue = 1.43;

	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_lms;
		sheet = "tech_params_lms";
	run;
	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_f;
		sheet = "tech_params_f";
	run;
	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_w;
		sheet = "tech_params_w";
	run;
	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_wg;
		sheet = "tech_params_wg";
	run;
	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_h;
		sheet = "tech_params_h";
	run;

/*C: Modeling Decisions*/
	%let enable_day_ahead_buyout = yes;

/*D: LOE Adjustment*/
proc import datafile = "&inpath.\LOE AFs for Net EAS.xlsx"
	out = loe
	DBMS = XLSX replace;
	sheet = "LOE";
run;
data in_loe;
	format DATE date10.;
	format TIME time8.;
	set loe;
	TIME = hms(HOUR,0,SECOND);
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 3. Merge Together Price Data for Each Hour								   			   	   ;
	/*A: Merge Together Emissions Prices*/
	/*B: Merge Together Fuel Prices*/
	/*C: Format Energy LBMP and LOE*/
	/*D: Combine All Datasets*/
	/*E: Set Proper Gas and Reserve Prices in Each Zone*/
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*Final format will be "Long" with one observation for each Zone-Hour*/

/*A: Merge Together Emissions Prices*/
/*Note: SO2 CSAPR includes SO CAIR Prior to 2015. The name is simply to distinguish from ARP allowances in this code*/
proc sql;
	create table prices_emissions as select
		coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE) as DATE format mmddyy10.,
    	a.CO2_PRICE,
		b.NOX_ANNUAL_PRICE,
		b.NOX_SEASONAL_PRICE,
		c.SO2_PRICE as SO2_CSAPR_PRICE,
		case when ~missing(e.SO2_PRICE) then e.SO2_PRICE else d.SO2_PRICE end as SO2_ARP_PRICE,
		dates.DATE as KEEP_DATES,
		dates.YEAR_NUMBER as MODEL_YEAR
	from in_dates as dates
		full outer join in_e_co2 as a
		on dates.DATE = a.AUCTION_DATE
		full outer join In_e_nox as b
		on coalesce(dates.DATE,a.AUCTION_DATE) = b.DATE
		full outer join In_e_so2_CSAPR as c
		on coalesce(dates.DATE,a.AUCTION_DATE,b.DATE) = c.DATE
		left join In_e_so2_arp_epa as d
		on year(coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE)) = d.DATE
		left join In_e_so2_arp as e
		on coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE) = e.DATE
	order by DATE;
quit;

/*Carry forward previous price when missing*/
data prices_emissions2 (drop = OLD: KEEP_DATES);
	retain DATE CO2_PRICE NOX_ANNUAL_PRICE NOX_SEASONAL_PRICE SO2_CSAPR_PRICE SO2_ARP_PRICE;
	set prices_emissions (rename = (CO2_PRICE=OLD_CO2_PRICE NOX_ANNUAL_PRICE=OLD_NOX_ANNUAL_PRICE NOX_SEASONAL_PRICE=OLD_NOX_SEASONAL_PRICE SO2_CSAPR_PRICE=OLD_SO2_CSAPR_PRICE SO2_ARP_PRICE=OLD_SO2_ARP_PRICE));
	if ~missing(OLD_CO2_PRICE) 			then CO2_PRICE = 			OLD_CO2_PRICE;
	if ~missing(OLD_NOX_ANNUAL_PRICE) 	then NOX_ANNUAL_PRICE = 	OLD_NOX_ANNUAL_PRICE;
	if ~missing(OLD_NOX_SEASONAL_PRICE) then NOX_SEASONAL_PRICE = 	OLD_NOX_SEASONAL_PRICE;
	if ~missing(OLD_SO2_CSAPR_PRICE) 	then SO2_CSAPR_PRICE= 		OLD_SO2_CSAPR_PRICE;
	if ~missing(OLD_SO2_ARP_PRICE) 		then SO2_ARP_PRICE= 		OLD_SO2_ARP_PRICE;

	/*Drop leading/lagging rows*/
	if ~missing(KEEP_DATES);
run;

/*B: Merge Together Fuel Prices*/
proc sql;
	create table prices_fuel as select 
		coalesce(dates.DATE, a.DATE, b.DAY) as DATE format mmddyy10.,
		a.Transco_Z6_Price,
		a.TGP_Z6_Price,
		a.TETCO_M3_Price,
		a.Iroquois_Z2_Price,
		a.Dominion_N_Price,
		a.Dominion_S_Price,
		a.Millennium_Price,
		/*Convert units from $/gallon to $/MMBtu using EIA conversion factor*/
		/*of 137,381 BTU/gallon												*/
		b.U_S__No_2_Diesel_Ultra_Low_Sulfu * 7.2790269396787 as ULSD_PRICE
	from in_dates as dates
		full outer join in_f_gas_lag as a
		on dates.DATE = a.DATE
		full outer join in_f_oil as b
		on coalesce(dates.DATE, a.DATE) = b.DAY
	order by DATE;
quit;

/*2020.09.21 Update: Carry backwards next fuel price when missing*/
proc sort data = prices_fuel;
      by descending DATE;
run;
data prices_fuel_fill (rename = (ULSD_PRICE_FILL = ULSD_PRICE DOMINION_N_PRICE_FILL = DOMINION_N_PRICE DOMINION_S_PRICE_FILL = DOMINION_S_PRICE IROQUOIS_Z2_PRICE_FILL = IROQUOIS_Z2_PRICE MILLENNIUM_PRICE_FILL = MILLENNIUM_PRICE TETCO_M3_PRICE_FILL = TETCO_M3_PRICE TGP_Z6_PRICE_FILL = TGP_Z6_PRICE TRANSCO_Z6_PRICE_FILL = TRANSCO_Z6_PRICE));
	set prices_fuel;
	retain DOMINION_N_PRICE_FILL DOMINION_S_PRICE_FILL IROQUOIS_Z2_PRICE_FILL 
		   MILLENNIUM_PRICE_FILL TETCO_M3_PRICE_FILL TGP_Z6_PRICE_FILL 
		   TRANSCO_Z6_PRICE_FILL ULSD_PRICE_FILL;
	by descending DATE;

	if ~missing(DOMINION_N_PRICE) then DOMINION_N_PRICE_FILL = DOMINION_N_PRICE;
	if ~missing(DOMINION_S_PRICE) then DOMINION_S_PRICE_FILL = DOMINION_S_PRICE;
	if ~missing(IROQUOIS_Z2_PRICE) then IROQUOIS_Z2_PRICE_FILL = IROQUOIS_Z2_PRICE;
	if ~missing(MILLENNIUM_PRICE) and MILLENNIUM_PRICE ~= "NA" then MILLENNIUM_PRICE_FILL = input(MILLENNIUM_PRICE, best.);
	if ~missing(TETCO_M3_PRICE) then TETCO_M3_PRICE_FILL = TETCO_M3_PRICE;
	if ~missing(TGP_Z6_PRICE) then TGP_Z6_PRICE_FILL = TGP_Z6_PRICE;
	if ~missing(TRANSCO_Z6_PRICE) then TRANSCO_Z6_PRICE_FILL = TRANSCO_Z6_PRICE;
	if ~missing(ULSD_PRICE) then ULSD_PRICE_FILL = ULSD_PRICE;

	drop DOMINION_N_PRICE DOMINION_S_PRICE IROQUOIS_Z2_PRICE MILLENNIUM_PRICE TETCO_M3_PRICE TGP_Z6_PRICE TRANSCO_Z6_PRICE ULSD_PRICE;
run;

proc sort data = prices_fuel_fill;
      by DATE;
run;

/*C: Format LOE*/
proc transpose data = in_loe
	out = in_loe_long (rename=(Col1=LOE)) name=Zone;
	by DATE TIME;
	var CAPITL CENTRL HUD_VL LONGIL NYC;
run;

/*D: Combine All Datasets*/
data merge_base;
	set in_lbmp_dam_adj;
	format AREA $2.;

	if zone = "CENTRL" then AREA = "C";
	else if zone = "CAPITL" then AREA = "F";
	else if zone = "HUD_VL" then AREA = "G";
	else if zone = "NYC" then AREA = "J";
	else if zone = "LONGIL" then AREA = "K";

	if AREA in ("C", "F") then CAPACITY_ZONE = "ROS";
	else if AREA in ("G") then CAPACITY_ZONE = "HV";
	else if AREA in ("J") then CAPACITY_ZONE = "NYC";
	else if AREA in ("K") then CAPACITY_ZONE = "LI";

	/*Duplicate Zone G for Rockland/Dutchess Permutations*/
	if zone = "HUD_VL" then do;
		AREA = "G1"; output;
		AREA = "G2"; output;
	end;
	else output;
run;

proc sql;
	create table merge_base2 as select
    	a.DATE label = 'Date', 
		a.TIME label = 'Time', 
		g.DATE as KEEP_DATES label = 'Flag for Dates in the Current Modeling Period',
		g.MODEL_YEAR label = 'Numeric Year in 3-Year Modeling Period (1, 2 or 3)',
		a.ZONE label = 'Zone Name', 
		a.AREA label = 'Zone Letter (A-K; G1 = Dutchess; G2 = Rockland)', 
		a.CAPACITY_ZONE label = 'Capacity Zone (LI, NYC, HV, ROS)',

		b.DAM_LBMP label = 'Day-Ahead Hourly Average LBMP',
		c.RTD_LBMP label = 'Real-Time Dispatch Hourly Average LBMP',

		d.DAM_30MIN label = 'Day-Ahead Hourly Average 30-Minute Reserve Price',
		d.DAM_10MIN_NONSPIN label = 'Day-Ahead Hourly Average 10-Minute Nonspin Reserve Price',

		e.RTD_30MIN label = 'Real-Time Dispatch Hourly Average 30-Minute Reserve Price',
		e.RTD_10MIN_NONSPIN label = 'Real-Time Dispatch Hourly Average 10-Minute Nonspin Reserve Price',

		loe.LOE label = 'Level of Excess Scaling Factor',

		f.TRANSCO_Z6_PRICE label = 'Transco Z6 NY Natural Gas Price ($/MMBtu)',
		f.TGP_Z6_PRICE label = 'TGP Z6 Natural Gas Price ($/MMBtu)',
		f.TETCO_M3_PRICE label = 'TETCO-M3 Natural Gas Price ($/MMBtu)',
		f.IROQUOIS_Z2_PRICE label = 'Iroquois Z2 Natural Gas Price ($/MMBtu)',
		f.DOMINION_N_PRICE label = 'Dominion North Natural Gas Price ($/MMBtu)',
		f.DOMINION_S_PRICE label = 'Dominion South Natural Gas Price ($/MMBtu)',
		f.ULSD_PRICE label = 'ULSD Oil Price ($/MMBtu)',

		g.CO2_PRICE label = 'RGGI CO2 Allowance Price ($/ton)',
		g.NOX_ANNUAL_PRICE label = 'CAIR/CSAPR NOX Annual Allowance Price ($/allowance)',
		g.NOX_SEASONAL_PRICE label = 'CAIR/CSAPR NOX Seasonal Allowance Price ($/allowance)',
		g.SO2_CSAPR_PRICE label = 'CAIR/CSAPR SO2 Allowance Price ($/allowance)',
		g.SO2_ARP_PRICE label = 'Acid Rain Program SO2 Allowance Price ($/allowance)',

		rs.RATE as RS1 label = 'Rate Schedule 1 Cost Adder ($/MWh)',

		/*Unit Parameters - LMS*/
		/*Oil emissions modeled based on Winter emissions in all seasons. We thus use Winter paramaters below.*/
		case when month(a.DATE) in (5,6,7,8,9,10) then h.HR_SUM_LMS / 1000		else h.HR_WIN_LMS  / 1000	end as HR_LMS label = 'LMS Heat Rate (MMBtu/MWh)',
		h.VOC_NG_LMS label = 'LMS Gas-Fired Variable Operating Cost ($/MWh)',
		h.VOC_OIL_LMS label = 'LMS Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.ICAP_SUM_LMS 		else h.ICAP_WIN_LMS end as ICAP_LMS label = 'LMS ICAP (MW)',
		h.STARTUP_FUEL_LMS label = 'LMS Startup Fuel Burn (MMBtu)',
		h.STARTUP_VOM_LMS label = 'LMS Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.CO2_NG_SUM_LMS  / h.ICAP_SUM_LMS / 2000	else h.CO2_NG_WIN_LMS  / h.ICAP_WIN_LMS / 2000	end as CO2_NG_LMS label = 'LMS Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.CO2_OIL_SUM_LMS / h.ICAP_WIN_LMS / 2000	else h.CO2_OIL_WIN_LMS / h.ICAP_WIN_LMS / 2000	end as CO2_OIL_LMS label = 'LMS Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.NOX_NG_SUM_LMS  / h.ICAP_SUM_LMS / 2000	else h.NOX_NG_WIN_LMS  / h.ICAP_WIN_LMS / 2000	end as NOX_NG_LMS label = 'LMS Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.NOX_OIL_SUM_LMS / h.ICAP_WIN_LMS / 2000	else h.NOX_OIL_WIN_LMS / h.ICAP_WIN_LMS / 2000	end as NOX_OIL_LMS label = 'LMS Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.SO2_NG_SUM_LMS  / h.ICAP_SUM_LMS / 2000	else h.SO2_NG_WIN_LMS  / h.ICAP_WIN_LMS / 2000	end as SO2_NG_LMS label = 'LMS Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then h.SO2_OIL_SUM_LMS / h.ICAP_WIN_LMS / 2000	else h.SO2_OIL_WIN_LMS / h.ICAP_WIN_LMS / 2000	end as SO2_OIL_LMS label = 'LMS Oil-Fired SO2 Emissions (lb/hr)',
		h.EFORd_LMS label = 'LMS EFORd (%)',

		/*Unit Parameters - F*/
		case when month(a.DATE) in (5,6,7,8,9,10) then i.HR_SUM_F / 1000		else i.HR_WIN_F  / 1000	end as HR_F label = 'Frame Heat Rate (MMBtu/MWh)',
		i.VOC_NG_F label = 'Frame Gas-Fired Variable Operating Cost ($/MWh)',
		i.VOC_OIL_F label = 'Frame Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.ICAP_SUM_F 		else i.ICAP_WIN_F end as ICAP_F label = 'Frame ICAP (MW)',
		i.STARTUP_FUEL_F label = 'Frame Startup Fuel Burn (MMBtu)',
		i.STARTUP_VOM_F label = 'Frame Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.CO2_NG_SUM_F  / i.ICAP_SUM_F / 2000	else i.CO2_NG_WIN_F  / i.ICAP_WIN_F / 2000	end as CO2_NG_F label = 'Frame Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.CO2_OIL_SUM_F / i.ICAP_WIN_F / 2000	else i.CO2_OIL_WIN_F / i.ICAP_WIN_F / 2000	end as CO2_OIL_F label = 'Frame Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.NOX_NG_SUM_F  / i.ICAP_SUM_F / 2000	else i.NOX_NG_WIN_F  / i.ICAP_WIN_F / 2000	end as NOX_NG_F label = 'Frame Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.NOX_OIL_SUM_F / i.ICAP_WIN_F / 2000	else i.NOX_OIL_WIN_F / i.ICAP_WIN_F / 2000	end as NOX_OIL_F label = 'Frame Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.SO2_NG_SUM_F  / i.ICAP_SUM_F / 2000	else i.SO2_NG_WIN_F  / i.ICAP_WIN_F / 2000	end as SO2_NG_F label = 'Frame Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then i.SO2_OIL_SUM_F / i.ICAP_WIN_F / 2000	else i.SO2_OIL_WIN_F / i.ICAP_WIN_F / 2000	end as SO2_OIL_F label = 'Frame Oil-Fired SO2 Emissions (lb/hr)',
		i.EFORd_F label = 'Frame EFORd (%)',

		/*Unit Parameters - Wartsila*/
		case when month(a.DATE) in (5,6,7,8,9,10) then j.HR_SUM_W / 1000		else j.HR_WIN_W  / 1000	end as HR_W label = 'Wartsila Heat Rate (MMBtu/MWh)',
		j.VOC_NG_W label = 'Wartsila Gas-Fired Variable Operating Cost ($/MWh)',
		j.VOC_OIL_W label = 'Wartsila Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.ICAP_SUM_W 		else j.ICAP_WIN_W end as ICAP_W label = 'Wartsila ICAP (MW)',
		j.STARTUP_FUEL_W label = 'Wartsila Startup Fuel Burn (MMBtu)',
		j.STARTUP_VOM_W label = 'Wartsila Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.CO2_NG_SUM_W  / j.ICAP_SUM_W / 2000	else j.CO2_NG_WIN_W  / j.ICAP_WIN_W / 2000	end as CO2_NG_W label = 'Wartsila Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.CO2_OIL_SUM_W / j.ICAP_WIN_W / 2000	else j.CO2_OIL_WIN_W / j.ICAP_WIN_W / 2000	end as CO2_OIL_W label = 'Wartsila Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.NOX_NG_SUM_W  / j.ICAP_SUM_W / 2000	else j.NOX_NG_WIN_W  / j.ICAP_WIN_W / 2000	end as NOX_NG_W label = 'Wartsila Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.NOX_OIL_SUM_W / j.ICAP_WIN_W / 2000	else j.NOX_OIL_WIN_W / j.ICAP_WIN_W / 2000	end as NOX_OIL_W label = 'Wartsila Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.SO2_NG_SUM_W  / j.ICAP_SUM_W / 2000	else j.SO2_NG_WIN_W  / j.ICAP_WIN_W / 2000	end as SO2_NG_W label = 'Wartsila Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j.SO2_OIL_SUM_W / j.ICAP_WIN_W / 2000	else j.SO2_OIL_WIN_W / j.ICAP_WIN_W / 2000	end as SO2_OIL_W label = 'Wartsila Oil-Fired SO2 Emissions (lb/hr)',
		j.EFORd_W label = 'Wartsila EFORd (%)',

		/*Unit Parameters - Wartsila Gas Only*/
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.HR_SUM_WG / 1000		else j2.HR_WIN_WG  / 1000	end as HR_WG label = 'Wartsila Gas Only Heat Rate (MMBtu/MWh)',
		j2.VOC_NG_WG label = 'Wartsila Gas Only Gas-Fired Variable Operating Cost ($/MWh)',
		j2.VOC_OIL_WG label = 'Wartsila Gas Only Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.ICAP_SUM_WG 		else j2.ICAP_WIN_WG end as ICAP_WG label = 'Wartsila Gas Only ICAP (MW)',
		j2.STARTUP_FUEL_WG label = 'Wartsila Gas Only Startup Fuel Burn (MMBtu)',
		j2.STARTUP_VOM_WG label = 'Wartsila Gas Only Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.CO2_NG_SUM_WG  / j2.ICAP_SUM_WG / 2000	else j2.CO2_NG_WIN_WG  / j2.ICAP_WIN_WG / 2000	end as CO2_NG_WG label = 'Wartsila Gas Only Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.CO2_OIL_SUM_WG / j2.ICAP_WIN_WG / 2000	else j2.CO2_OIL_WIN_WG / j2.ICAP_WIN_WG / 2000	end as CO2_OIL_WG label = 'Wartsila Gas Only Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.NOX_NG_SUM_WG  / j2.ICAP_SUM_WG / 2000	else j2.NOX_NG_WIN_WG  / j2.ICAP_WIN_WG / 2000	end as NOX_NG_WG label = 'Wartsila Gas Only Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.NOX_OIL_SUM_WG / j2.ICAP_WIN_WG / 2000	else j2.NOX_OIL_WIN_WG / j2.ICAP_WIN_WG / 2000	end as NOX_OIL_WG label = 'Wartsila Gas Only Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.SO2_NG_SUM_WG  / j2.ICAP_SUM_WG / 2000	else j2.SO2_NG_WIN_WG  / j2.ICAP_WIN_WG / 2000	end as SO2_NG_WG label = 'Wartsila Gas Only Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then j2.SO2_OIL_SUM_WG / j2.ICAP_WIN_WG / 2000	else j2.SO2_OIL_WIN_WG / j2.ICAP_WIN_WG / 2000	end as SO2_OIL_WG label = 'Wartsila Gas Only Oil-Fired SO2 Emissions (lb/hr)',
		j2.EFORd_WG label = 'Wartsila Gas Only EFORd (%)',

		/*Unit Parameters - H Machine*/
		case when month(a.DATE) in (5,6,7,8,9,10) then k.HR_SUM_H / 1000						else k.HR_WIN_H  / 1000	end as HR_H label = 'H Machine Heat Rate (MMBtu/MWh)',
		k.VOC_NG_H label = 'H Machine Gas-Fired Variable Operating Cost ($/MWh)',
		k.VOC_OIL_H label = 'H Machine Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.ICAP_SUM_H 							else k.ICAP_WIN_H end as ICAP_H label = 'H Machine ICAP (MW)',
		k.STARTUP_FUEL_H label = 'H Machine Startup Fuel Burn (MMBtu)',
		k.STARTUP_VOM_H label = 'H Machine Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.CO2_NG_SUM_H  / k.ICAP_SUM_H / 2000	else k.CO2_NG_WIN_H  / k.ICAP_WIN_H / 2000	end as CO2_NG_H label = 'H Machine Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.CO2_OIL_SUM_H / k.ICAP_WIN_H / 2000	else k.CO2_OIL_WIN_H / k.ICAP_WIN_H / 2000	end as CO2_OIL_H label = 'H Machine Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.NOX_NG_SUM_H  / k.ICAP_SUM_H / 2000	else k.NOX_NG_WIN_H  / k.ICAP_WIN_H / 2000	end as NOX_NG_H label = 'H Machine Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.NOX_OIL_SUM_H / k.ICAP_WIN_H / 2000	else k.NOX_OIL_WIN_H / k.ICAP_WIN_H / 2000	end as NOX_OIL_H label = 'H Machine Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.SO2_NG_SUM_H  / k.ICAP_SUM_H / 2000	else k.SO2_NG_WIN_H  / k.ICAP_WIN_H / 2000	end as SO2_NG_H label = 'H Machine Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then k.SO2_OIL_SUM_H / k.ICAP_WIN_H / 2000	else k.SO2_OIL_WIN_H / k.ICAP_WIN_H / 2000	end as SO2_OIL_H label = 'H Machine Oil-Fired SO2 Emissions (lb/hr)',
		k.EFORd_H label = 'H Machine EFORd (%)'

	from merge_base as a
		left join In_lbmp_dam_adj as b
			on a.DATE = b.DATE and a.TIME = b.TIME and a.ZONE = b.ZONE
		left join In_lbmp_rtd_adj as c
			on a.DATE = c.DATE and a.TIME = c.TIME and a.ZONE = c.ZONE
		left join In_anc_dam_adj as d
			on a.DATE = d.DATE and a.TIME = d.TIME and a.ZONE = d.ZONE
		left join In_anc_rtd_adj as e
			on a.DATE = e.DATE and a.TIME = e.TIME and a.ZONE = e.ZONE 
		left join Prices_fuel_fill as f
			on a.DATE = f.DATE
		left join Prices_emissions2 as g
			on a.DATE = g.DATE
		left join in_rs1 as rs
			on year(a.DATE) = rs.YEAR
		left join IN_LOE_LONG as loe
			on a.DATE = loe.DATE and a.TIME = loe.TIME and a.ZONE = loe.ZONE
		inner join in_tech_lms as h
			on a.AREA = h.AREA
		inner join in_tech_f as i
			on a.AREA = i.AREA
		inner join in_tech_w as j
			on a.AREA = j.AREA
		inner join in_tech_wg as j2
			on a.AREA = j2.AREA
		inner join in_tech_h as k
			on a.AREA = k.AREA
	order by AREA, DATE, TIME;
quit;

/*E: Set Proper Gas in Each Zone*/
data merge_data;
	set merge_base2;

	/*Assign Gas Prices to each Area*/
	if AREA in ("C") and ~missing(TETCO_M3_PRICE) then do;
		DAM_GAS_PRICE = TETCO_M3_PRICE + &ros_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (TETCO_M3_PRICE*&ros_fuel_prem.) + &ros_fuel_trans.;
		RTD_GAS_PRICE_SELL = (TETCO_M3_PRICE*&ros_fuel_disc.) + &ros_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &ros_ulsd_trans.;
	end;
	if AREA in ("F") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &ros_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&ros_fuel_prem.) + &ros_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&ros_fuel_disc.) + &ros_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &ros_ulsd_trans.;
	end;
	else if AREA in ("G2") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &hv_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&hv_fuel_prem.) + &hv_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&hv_fuel_disc.) + &hv_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &hv_ulsd_trans.;
	end;
	else if AREA in ("G1") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &hv_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&hv_fuel_prem.) + &hv_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&hv_fuel_disc.) + &hv_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &hv_ulsd_trans.;
	end;
	else if AREA in ("J") and ~missing(TRANSCO_Z6_PRICE) then do;
		DAM_GAS_PRICE = (TRANSCO_Z6_PRICE*&nyc_gas_tax.) + &nyc_fuel_trans.;
		RTD_GAS_PRICE_BUY  = ((TRANSCO_Z6_PRICE*&nyc_fuel_prem.)*&nyc_gas_tax.) + &nyc_fuel_trans.;
		RTD_GAS_PRICE_SELL = ((TRANSCO_Z6_PRICE*&nyc_fuel_disc.)*&nyc_gas_tax.) + &nyc_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = (ULSD_PRICE*&nyc_sales_tax.) + &nyc_ulsd_trans.;
	end;
	else if AREA in ("K") and ~missing(TRANSCO_Z6_PRICE) then do;
		DAM_GAS_PRICE = (TRANSCO_Z6_PRICE*&li_gas_tax.) + &li_fuel_trans.;
		RTD_GAS_PRICE_BUY  = ((TRANSCO_Z6_PRICE*&li_fuel_prem.)*&li_gas_tax.) + &li_fuel_trans.;
		RTD_GAS_PRICE_SELL = ((TRANSCO_Z6_PRICE*&li_fuel_disc.)*&li_gas_tax.) + &li_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &li_ulsd_trans.;
	end;

run;

/*Carry forward fuel prices when missing*/
proc sort data = merge_data;
      by AREA DATE TIME;
run;

data netEAS.full_input_data (drop = KEEP_DATES);
	set merge_data;

	if missing(CO2_PRICE) then CO2_PRICE = 0;
	if missing(SO2_CSAPR_PRICE) then SO2_CSAPR_PRICE = 0;
	if missing(SO2_ARP_PRICE) then SO2_ARP_PRICE = 0;
	if missing(NOX_ANNUAL_PRICE) then NOX_ANNUAL_PRICE = 0;
	if missing(NOX_SEASONAL_PRICE) then NOX_SEASONAL_PRICE = 0;
	if missing(RS1) then RS1 = 0;
	if missing(LOE) then LOE = 1;

	/*During NOX Season, plant owners must purchase both Seasonal and Annual Credits*/
	if month(DATE) in (5,6,7,8,9) then NOX_PRICE = NOX_ANNUAL_PRICE+NOX_SEASONAL_PRICE;
	else NOX_PRICE = NOX_ANNUAL_PRICE;

	/*Units must comply with both SO2 CAIR/CSARR and ARP Regulations*/
	/*From 2010-2014, CAIR allowances only covered half a ton of emissions, effectively doubling the price to cover one ton of emissions*/
	if year(DATE) in (2010,2011,2012,2013,2014) then SO2_PRICE = (2*SO2_CSAPR_PRICE)+SO2_ARP_PRICE;
	else SO2_PRICE = (1*SO2_CSAPR_PRICE)+SO2_ARP_PRICE;

	if ~missing(KEEP_DATES);

	label DAM_GAS_PRICE = 'Final Day-Ahead Natural Gas Price ($/MMBtu)';
	label RTD_GAS_PRICE_BUY = 'Final Real-Time Dispatch Natural Gas Price For Unit Purchase with Premium ($/MMBtu)';
	label RTD_GAS_PRICE_SELL = 'Final Real-Time Dispatch Natural Gas Price For Unit Buyout at Discount ($/MMBtu)';
	label ULSD_PRICE =  'ULSD Oil Price ($/MMBtu)';
	label NOX_PRICE = 'Total NOX Allowance Price ($/ton)';
	label SO2_PRICE = 'Total SO2 Allowance Price ($/ton)';
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 4. Calculate Base DAM and RTD Energy Profits in Each Hour					   			   	   ;
* 	 A: Decide to run on Oil vs. Gas;
*	 B: Calculate Potential Real-Time Fuel Penalty;
* 	 C: Determine Proper Reserve Type for Unit (10-min spinning or not);
*    D: Calculate Hourly Energy Profit and Total Cost Prior to Startup Costs by Zone;
*    E: Calculate Startup Costs if Unit Starts in Given Hour;
*    NOTE: All fuel calculations also calculate the 'loss' if a unit is to buy-out of the
		commitment later and not use the fuel												   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*Calculate potential hourly profit in DAM Energy and Reserve*/
%macro eas_profits(tech=);
	/*Adjust "tech" for the Wartsila Gas Only Case, which requires different unit parameters*/
	%if &tech. = W and &enable_gas_only. = yes %then %do;
		%put Tech was &tech. in gas only mode;
		%let tech = WG;
		%put Tech updated to &tech.;
	%end;

	/*A: Decide to run on Oil vs. Gas*/
	data hr_profits_fuel_&tech.;
		set netEAS.full_input_data;
		/*LOE Disable*/
		%if &enable_LOE. ~= yes %then %do; LOE = 1; %end;

		/*If gas only, set impossibly high oil price and drop NYC and LI*/
		%if &enable_gas_only. = yes %then %do; 
			ULSD_PRICE = 99999;
			if AREA in ('J','K') then delete; 
		%end;

		/*Gas*/
		DAM_GAS_FUEL_COST = (HR_&tech. * DAM_GAS_PRICE); label DAM_GAS_FUEL_COST = 'Day-Ahead Gas Cost ($/MWh)';
		RTD_GAS_FUEL_COST = (HR_&tech. * RTD_GAS_PRICE_BUY); label RTD_GAS_FUEL_COST = 'Real-Time Gas Cost ($/MWh)';
		GAS_EMISSIONS_COST = (CO2_NG_&tech.*CO2_PRICE) + (NOX_NG_&tech.*NOX_PRICE) + (SO2_NG_&tech.*SO2_PRICE); label GAS_EMISSIONS_COST = 'Gas Emissions Cost ($/MWh)';
		DAM_GAS_COST = DAM_GAS_FUEL_COST + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST; label DAM_GAS_COST = 'Total Day-Ahead Cost when Gas-Fired ($/MWh)';
		RTD_GAS_COST = RTD_GAS_FUEL_COST + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST; label RTD_GAS_COST = 'Total Real-Time Cost when Gas-Fired ($/MWh)';

		/*Oil*/
		OIL_FUEL_COST = (HR_&tech. * ULSD_PRICE); label OIL_FUEL_COST = 'Oil Cost ($/MWh)';
		OIL_EMISSIONS_COST = (CO2_OIL_&tech.*CO2_PRICE) + (NOX_OIL_&tech.*NOX_PRICE) + (SO2_OIL_&tech.*SO2_PRICE); label OIL_EMISSIONS_COST = 'Oil Emissions Cost ($/MWh)';
		OIL_COST = OIL_FUEL_COST + VOC_OIL_&tech. + RS1 + OIL_EMISSIONS_COST; label OIL_COST = 'Total Cost when Oil-Fired ($/MWh)';

		/*Select between fuels using day-ahead gas price*/
		if OIL_COST < DAM_GAS_COST and ~missing(ULSD_PRICE) then do;
			ECONOMIC_FUEL = "OIL";
			DAM_ECONOMIC_FUEL_PRICE = ULSD_PRICE;
			RTD_ECONOMIC_FUEL_PRICE = ULSD_PRICE;
			DAM_COST = OIL_COST;
			RTD_COST = OIL_COST;
		end; else do;
			ECONOMIC_FUEL = "GAS"; label ECONOMIC_FUEL = 'Day-Ahead Economic Fuel Type';
			DAM_ECONOMIC_FUEL_PRICE = DAM_GAS_PRICE; label DAM_ECONOMIC_FUEL_PRICE = 'Total Day-Ahead Fuel Price ($/MMBtu)';
			RTD_ECONOMIC_FUEL_PRICE = RTD_GAS_PRICE_BUY; label RTD_ECONOMIC_FUEL_PRICE = 'Total Real-Time Fuel Price ($/MMBtu)';
			DAM_COST = DAM_GAS_COST; label DAM_COST = 'Total Day-Ahead Cost ($/MWh)';
			RTD_COST = RTD_GAS_COST; label RTD_COST = 'Total Real-Time Cost ($/MWh)';
		end;

		/*Calculate fuel losses if unit buys-out of day-ahead fuel commitment*/
		if ECONOMIC_FUEL = "GAS" then E_BUYOUT_FUEL_LOSS = DAM_GAS_PRICE - RTD_GAS_PRICE_SELL;
		else if ECONOMIC_FUEL = "OIL" then E_BUYOUT_FUEL_LOSS = 0;
		label E_BUYOUT_FUEL_LOSS = '$ Loss per MMBtu from Day-Ahead Fuel Commitment Buyout ($/MMBtu)';
	run;

	/*Calculate Potential Energy/Reserve Profits in Each Hour (and Startup Costs)*/
	data hr_profits_dam_&tech.;
		set hr_profits_fuel_&tech.;

	/*   B: Calculate Potential Real-Time Fuel Penalty*/
		E_BUYOUT_FUEL_LOSS_COST = (HR_&tech. * E_BUYOUT_FUEL_LOSS); label E_BUYOUT_FUEL_LOSS_COST = '$ Loss per MWh from Day-Ahead Fuel Commitment Buyout ($/MWh)';

	/* 	 C: Determine Proper Reserve Type for Unit (10-min spinning or not)*/
		if &&ns10min_&tech. = "Yes" then do;
			/* In order to commit DAM reserves, a unit must secure fuel even though it does not intend to use the fuel. As a result, the cost
					of a day-ahead reserve commitment can be approximated as the losses due to unused fuel that must be sold back to a pipeline.
					This logic does not apply a cost for oil, since the "E_BUYOUT_FUEL_LOSS" variable is set to 0 when oil is the economic fuel.*/
			DAM_HR_RESERVE_PROFIT = LOE*DAM_10MIN_NONSPIN - E_BUYOUT_FUEL_LOSS_COST;
			/* Real-time reserve cost depends on day-ahead commitment (e.g., has fuel already been secured?). This is addressed later in the code.*/
			RTD_HR_RESERVE_PRICE = LOE*RTD_10MIN_NONSPIN;
		end;
		else do;
			DAM_HR_RESERVE_PROFIT = LOE*DAM_30MIN - E_BUYOUT_FUEL_LOSS_COST;
			RTD_HR_RESERVE_PRICE = LOE*RTD_30MIN;
		end;
		label DAM_HR_RESERVE_PROFIT = 'Final Day-Ahead Hourly Average Reserve Profit';
		label RTD_HR_RESERVE_PRICE = 'Final Real-Time Hourly Average Reserve P';

	/*    D: Calculate Hourly Energy Profit and Total Cost Prior to Startup Costs by Zone*/
		/*Hard cap at price of $4,000/MWh (transmission shortage cost maximum pricing)*/
		DAM_HR_ENERGY_PROFIT = (min(LOE*DAM_LBMP,4000) - DAM_COST); label DAM_HR_ENERGY_PROFIT = 'Day-Ahead Potential Energy Profit ($/MWh)';
		RTD_HR_ENERGY_PROFIT = (min(LOE*RTD_LBMP,4000) - RTD_COST); label RTD_HR_ENERGY_PROFIT = 'Real-Time Potential Energy Profit ($/MWh)';

	/*    E: Calculate Startup Costs if Unit Starts in Given Hour*/
		DAM_STARTUP_FUEL_COST = (STARTUP_FUEL_&tech. * DAM_ECONOMIC_FUEL_PRICE) / ICAP_&tech.; label DAM_STARTUP_FUEL_COST = 'Day-Ahead Fuel Cost to Startup in Hour ($/MWh)';
		DAM_STARTUP_VOM_COST = (STARTUP_VOM_&tech.) / ICAP_&tech.; label DAM_STARTUP_VOM_COST = 'Day-Ahead Variable Cost to Startup in Hour ($/MWh)';
		DAM_STARTUP_COST = DAM_STARTUP_FUEL_COST + DAM_STARTUP_VOM_COST; label DAM_STARTUP_COST = 'Total Day-Ahead Cost to Startup in Hour ($/MWh)';

		RTD_STARTUP_FUEL_COST = (STARTUP_FUEL_&tech. * RTD_ECONOMIC_FUEL_PRICE) / ICAP_&tech.; label RTD_STARTUP_FUEL_COST = 'Real-Time Fuel Cost to Startup in Hour ($/MWh)';
		RTD_STARTUP_VOM_COST = (STARTUP_VOM_&tech.) / ICAP_&tech.; label RTD_STARTUP_VOM_COST = 'Real-Time Variable Cost to Startup in Hour ($/MWh)';
		RTD_STARTUP_COST = RTD_STARTUP_FUEL_COST + RTD_STARTUP_VOM_COST; label RTD_STARTUP_COST = 'Total Real-Time Cost to Startup in Hour ($/MWh)';

		E_BUYOUT_STARTUP_FUEL_LOSSES = (STARTUP_FUEL_&tech. * (E_BUYOUT_FUEL_LOSS)) / ICAP_&tech.; label E_BUYOUT_STARTUP_FUEL_LOSSES = '$ Startup Fuel Loss per MWh from Day-Ahead Energy Commitment Buyout ($/MWh)';
	run;

	proc sort data = hr_profits_dam_&tech.;
	      by AREA DATE TIME;
	run;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 5. DAM Energy Blocks - Set Day-Ahead Behavior and Determine if Real-Time Buyout Profitable   ;
	* 	 A: Flag "Blocks" of hours with positive hourly energy profit (pre startup cost);
	* 	 B: Ensure LMS unit can recover all costs within 1-hour amortization period;
	* 	 C: Sum profit and cost components for each block to use in block-level commitment logic;
	* 	 D: Decide if unit will commit energy in day-ahead
			1. Energy Block Profits (with Startup) > 0
			2. AND Energy Block Profits > Reserve Profits over same block ;
	* 	 E: Decide if unit will stick to energy in real-time or buyout or provide reserves 
			NOTE: We assume a unit will not buy-out of a partial block to simplify calculations
			See logic as described in step D
	* 	 F: Merge block-level decision back on to hour-level data;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*	A: Flag "Blocks" of hours with positive hourly energy profit*/
	data blk_hrly_profits_dam_&tech. (drop = _:);
		set hr_profits_dam_&tech.;
		by AREA DATE TIME;
		retain _COUNTER;

		/*Select Profitable DAM Hours*/
		if DAM_HR_ENERGY_PROFIT > 0 then DAM_POTENTIAL_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT;

		/*Label DAM Run Blocks and Apply Start Costs*/
		/*Counter to Count Run Blocks*/
		if first.AREA then _COUNTER = 0;

		_LAG1 = lag(DAM_POTENTIAL_ENERGY_PROFIT);
		/*Account for Startup Cost in First Hour of Block*/
		if ~missing(DAM_POTENTIAL_ENERGY_PROFIT) and _LAG1 = . then do;
			_COUNTER + 1;
			DAM_POTENTIAL_ENERGY_PROFIT = DAM_POTENTIAL_ENERGY_PROFIT - DAM_STARTUP_COST;
		end;

		if ~missing(DAM_POTENTIAL_ENERGY_PROFIT) then DAM_POTENTIAL_BLOCK_NUMBER = _COUNTER;

		label DAM_POTENTIAL_ENERGY_PROFIT = 'Day-Ahead Potential Run Block Energy Profit ($/MWh)';
		label DAM_POTENTIAL_BLOCK_NUMBER = 'Day-Ahead Potential Run Block Number';
	run;

	/*	B: Ensure LMS and Wartsila units can recover all costs within 1-hour amortization period*/
	%macro DAM_amort();
	%if &tech. = LMS or &tech. = W %then %do;
		data blk_hrly_profits_dam_&tech.2 (drop = LAG_T _COUNTER TRIGGER_STARTUP);
			set blk_hrly_profits_dam_&tech.;
			by AREA DATE TIME;
			retain TRIGGER_STARTUP;

			if first.AREA then _COUNTER = 0;
			/*Flag that unit to run in the first hour where costs are recovered*/ 
			if DAM_HR_ENERGY_PROFIT <= 0 then TRIGGER_STARTUP = .;
			else if DAM_HR_ENERGY_PROFIT - DAM_STARTUP_COST > 0 then TRIGGER_STARTUP = 1;

			LAG_T = lag1(TRIGGER_STARTUP);
			/*Create variable*/ 
			if LAG_T = . and TRIGGER_STARTUP = 1 then do;
				DAM_BLOCK_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT - DAM_STARTUP_COST;
				_COUNTER + 1;
			end;
			else if TRIGGER_STARTUP = 1 then DAM_BLOCK_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT;

			/*Count Block #s*/
			if ~missing(TRIGGER_STARTUP) then DAM_BLOCK_NUMBER = _COUNTER;

			label DAM_BLOCK_ENERGY_PROFIT = 'Day-Ahead Run Block Energy Profit ($/MWh)';
			label DAM_BLOCK_NUMBER = 'Day-Ahead Run Block Number';
		run;
	%end;
	/*F-CLASS AND H MACHINE DO NOT NEED 1-HR AMORT*/
	%else %do;
		data blk_hrly_profits_dam_&tech.2;
			set blk_hrly_profits_dam_&tech.;
			DAM_BLOCK_NUMBER = DAM_POTENTIAL_BLOCK_NUMBER;
			DAM_BLOCK_ENERGY_PROFIT = DAM_POTENTIAL_ENERGY_PROFIT;

			label DAM_BLOCK_ENERGY_PROFIT = 'Day-Ahead Run Block Energy Profit ($/MWh)';
			label DAM_BLOCK_NUMBER = 'Day-Ahead Run Block Number';
		run;
	%end;
	%mend;
	%DAM_amort;

	/* 	 C: Sum profit and cost components for each block to use in block-level commitment logic*/
	proc sql;
		create table blk_profits_dam_&tech. as select distinct
			AREA, DAM_BLOCK_NUMBER,
			count(DAM_BLOCK_NUMBER) as DAM_BLOCK_HOURS label = 'Hours in Day-Ahead Block',
			sum(DAM_BLOCK_ENERGY_PROFIT) as DAM_BLOCK_ENERGY_PROFIT label = 'Day-Ahead Block Energy Profit ($)',
			sum(DAM_HR_RESERVE_PROFIT)  as DAM_BLOCK_RESERVE_PROFIT label = 'Day-Ahead Block Reserve Profit ($)',
			sum(LOE*DAM_LBMP) as BLOCK_DAM_LBMP label = 'Day-Ahead Block LBMP ($)',
			sum(LOE*RTD_LBMP) as BLOCK_RTD_LBMP label = 'Real-Time Block LBMP ($)',
			sum(RTD_HR_RESERVE_PRICE) as BLOCK_RTD_RESERVE label = 'Real-Time Block Reserve Price ($)',
			sum(DAM_COST) + case when DATE = min(DATE) then DAM_STARTUP_COST else . end as BLOCK_COSTS label = 'Day-Ahead Block Costs ($)',
			sum(E_BUYOUT_FUEL_LOSS_COST) + max(E_BUYOUT_STARTUP_FUEL_LOSSES) as BUYOUT_FUEL_LOSSES label = '$ Loss per Block from Day-Ahead Energy Fuel Commitment Buyout ($)'
		from blk_hrly_profits_dam_&tech.2
		where ~missing(DAM_BLOCK_NUMBER)
		group by AREA, DAM_BLOCK_NUMBER
		having ~missing(BLOCK_COSTS)
		order by AREA, DAM_BLOCK_NUMBER;
	quit;

	/* 	 D: Decide if unit will commit energy in day-ahead
			1. Energy Block Profits (with Startup) > 0
			2. AND Energy Block Profits > Reserve Profits over same block */
	data blk_profits_dam_&tech.2;
		set blk_profits_dam_&tech.;

		/*Decide if DAM will Commit Energy*/
		format DAM_COMMIT $7.;
		if DAM_BLOCK_ENERGY_PROFIT > 0 and DAM_BLOCK_ENERGY_PROFIT > DAM_BLOCK_RESERVE_PROFIT then do;
			DAM_COMMIT = "ENERGY";
		end;
		label DAM_COMMIT = 'Day-Ahead Commitment (Energy/Reserve/None)';
	run;

	* 	 E: Decide if unit will stick to energy in real-time or buyout or provide reserves 
			NOTE: We assume a unit will not buy-out of a partial block to simplify calculations;
		/*Decide what unit will do for this block in RTD. A unit will either:
			Follow their DAM Schedule (ENERGY) [BLOCK_DAM_LBMP - BLOCK_COSTS]
			Buy-out of their DAM Schedule (BUYOUT) [BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES]
			Buy-out and commit RTD Reserves (RESERVES) [BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES + BLOCK_RTD_RESERVE]
		*/
		/*For final profits, allocate parts to 'other' and 'physical', where 'other' represnts
			profits due to other transactions (e.g., buyouts) and 'physical' represents profits
			due to physically providing product. Later in the code, EFORd will only apply to physical.*/
	data blk_profits_dam_&tech.3;
		set blk_profits_dam_&tech.2;
		format RTD_DISPATCH $7.;
		if DAM_COMMIT = "ENERGY" then do;
		%macro toggle_buyout();
			RTD_CHECK_ENERGY   = BLOCK_DAM_LBMP - BLOCK_COSTS;
			RTD_CHECK_BUYOUT   = BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES;
			/*Note that there is no separate RTD_RESERVE cost in the below calculation, as fuel has already been secured via the day-ahead commitment.*/
			RTD_CHECK_RESERVE  = BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES + BLOCK_RTD_RESERVE;

			/*Three potential profits if we allow for day-ahead buyout*/
			%if &enable_day_ahead_buyout. = yes %then %do;
				if (RTD_CHECK_ENERGY > RTD_CHECK_BUYOUT) and (RTD_CHECK_ENERGY > RTD_CHECK_RESERVE) then do;
					RTD_DISPATCH = "ENERGY";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
				end;
				else if (RTD_CHECK_BUYOUT >= RTD_CHECK_ENERGY) and (RTD_CHECK_BUYOUT >= RTD_CHECK_RESERVE) then do;
					RTD_DISPATCH = "BUYOUT";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_BUYOUT / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = RTD_CHECK_BUYOUT / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_PHYSICAL = 0;
				end;
				else if (RTD_CHECK_RESERVE > RTD_CHECK_ENERGY) and (RTD_CHECK_RESERVE > RTD_CHECK_BUYOUT) then do;
					RTD_DISPATCH = "RESERVE";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_RESERVE / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = (BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES) / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_PHYSICAL = BLOCK_RTD_RESERVE / DAM_BLOCK_HOURS;
				end;

			%end;
			/*If we don't allow for buyout behavior, simply set the dispatch to what was previously decided.*/
			%else %do;
					RTD_DISPATCH = "ENERGY";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
			%end;
		%mend;
		%toggle_buyout;

		label RTD_DISPATCH = 'Real-Time Dispatch (Energy/Reserve/None)';
		label RTD_CHECK_ENERGY = 'Block Profits if Unit Dispatches Energy ($)';
		label RTD_CHECK_BUYOUT = 'Block Profits if Unit Buys Out of Day-Ahead Energy Commitment ($)';
		label RTD_CHECK_RESERVE = 'Block Profits if Unit Buys Out of Day-Ahead Energy Commitment and Provides Reserves ($)';
		label RTD_FINAL_HRLY_PROFIT = 'Final Unit Profit in Each Hour Given Commitment Decision ($)';
		label RTD_FINAL_HRLY_OTHER = 'Other Component of Final Unit Profit in Each Hour Given Commitment Decision ($)';
		label RTD_FINAL_HRLY_PHYSICAL = 'Physical Component of Final Unit Profit in Each Hour Given Commitment Decision ($)';
		end;
	run;

	* 	 F: Merge block-level decision back on to hour-level data;
	proc sql;
		create table hr_profits_dam_&tech.2 as select
	    	a.*,
			b.DAM_COMMIT,
			b.RTD_DISPATCH,
			case when b.RTD_DISPATCH = "ENERGY" then a.ECONOMIC_FUEL else "" end as FIRED_ENERGY_FUEL label = 'Fuel Used in Energy Production (Gas/Oil)',
			b.RTD_FINAL_HRLY_PROFIT label = 'Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)',
			b.RTD_FINAL_HRLY_OTHER label = 'Other Component of Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)',
			b.RTD_FINAL_HRLY_PHYSICAL label = 'Physical Component of Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)'
		from blk_hrly_profits_dam_&tech.2 as a
			left join blk_profits_dam_&tech.3 as b
			on a.AREA = b.AREA and a.DAM_BLOCK_NUMBER = b.DAM_BLOCK_NUMBER
		order by a.AREA, a.DATE, a.TIME;
	quit;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 6. DAM Non-Energy Hours - Determine if Units Commit Reserves or No Commitment 		       ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/* For all other DAM hours, if the Reserve Profit > 0, the hour will be committed DAM Reserve*/
	data hr_profits_&tech.3;
		set hr_profits_dam_&tech.2;
		if missing(DAM_COMMIT) and DAM_HR_RESERVE_PROFIT > 0 then DAM_COMMIT = "RESERVE";
		else if missing(DAM_COMMIT) then DAM_COMMIT = "NONE";
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 7. RTD Energy Blocks - Determine if Profitable to Change Day-Ahead Non-Energy to 
							 Real-Time Energy Dispatch											   ;
	* NOTE: There are two ways a unit can decide to dispatch energy in real-time.
		1. By extending (in either direction) the length of a previous day-ahead commitment.
		2. A unit will start up for purely real-time energy dispatch if it can recover real-time
		   startup costs within 2 hours of starting.	;

	* NOTE: We solve this by evaluating potential real-time energy blocks. Blocks are formed by 
		 looking at hours where it would be profitable for a unit to buyout of a day-ahead 
		 reserve position or to commit if they had no day-ahead position. Hours already
		 committed via day-ahead are included in these blocks because we need to determine if
		 the unit has already budgeted for the unit to startup in a given block day-ahead.

	*    A. For each hour determine real-time energy profit left after buyout of day-ahead position;
	*    B. Flag potential real-time energy blocks;
	*    C. If any hour of the block was committed day-ahead, flag that startup cost has been paid ;
	*    D. For all other blocks, find first point (if any) where startup costs can be recovered 
			over a one (LMS/W) or two-hour (Frame) period. Flag all hours after this point for 
			energy dispatch																		   ;
	*    E. Update real time commitment to reflect flagged blocks ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*    A. For each hour determine real-time energy profit left after buyout of day-ahead position*/
	/*If from nothing, the RTD_ENERGY must be positive.
		If from Reserves, buying out + RTD_ENERGY must be positive*/

	data hr_profits_&tech.4 (drop = _:);
		set hr_profits_&tech.3;
		by AREA DATE TIME;
		retain _COUNTER _LAG1;

		if first.AREA then _COUNTER = 0;

		if DAM_COMMIT = "NONE" then RTD_POTENTIAL_ENERGY_PROFIT = RTD_HR_ENERGY_PROFIT;
		/*Below, the unit has already secured fuel day-ahead for its real-time commitment. We credit back the real time fuel premium that
			was previously included in RTD_HR_ENERGY_PROFIT through the RTD_COST*/
		if DAM_COMMIT = "RESERVE" then RTD_POTENTIAL_ENERGY_PROFIT = (RTD_HR_ENERGY_PROFIT + E_BUYOUT_FUEL_LOSS_COST) - RTD_HR_RESERVE_PRICE;
		if RTD_DISPATCH = "ENERGY" then RTD_POTENTIAL_ENERGY_PROFIT = RTD_FINAL_HRLY_PROFIT;

	/*    B. Flag potential real-time energy blocks*/
		_LAG1 = lag1(RTD_POTENTIAL_ENERGY_PROFIT);
		if RTD_POTENTIAL_ENERGY_PROFIT > 0 and _LAG1 <= 0 then _COUNTER + 1;

		if RTD_POTENTIAL_ENERGY_PROFIT < 0 then RTD_POTENTIAL_ENERGY_PROFIT = .;
		else RTD_POTENTIAL_BLOCK_NUMBER = _COUNTER;

		label RTD_POTENTIAL_ENERGY_PROFIT = 'Real-Time Potential Run Block Energy Profit ($/MWh)';
		label RTD_POTENTIAL_BLOCK_NUMBER = 'Real-Time Potential Run Block Number';
	run;

	/*    C. If any hour of the block was committed day-ahead, flag that startup cost has been paid 
		(already accounted for in RTD_FINAL_HRLY_PROFIT)*/
	proc sql;
		create table hr_profits_&tech.5 as select
			*,
			max(case when RTD_DISPATCH = "ENERGY" then 1 else . end) as RTD_BLOCK_STARTUP_RECOVERED_DAM label = 'Flag if the RTD Energy Block is Already Assured Start-Up Recovery via DAM Commitment'
		from hr_profits_&tech.4
		group by AREA, RTD_POTENTIAL_BLOCK_NUMBER
		order by AREA, DATE, TIME;
	quit;

	/*    D. For all other blocks, find first point (if any) where startup costs can be recovered 
			over a one (LMS/W) or two-hour (Frame/H) period. Flag all hours after this point for 
			energy dispatch*/
	data hr_profits_&tech.6 (drop = _:);
		set hr_profits_&tech.5;
		by AREA DATE TIME;
		retain _LAG1 RTD_BLOCK_HOUR;

		_LAG1 = lag1(RTD_POTENTIAL_ENERGY_PROFIT);

		/*Label hours in each block*/
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) and missing(_LAG1) then RTD_BLOCK_HOUR = 0;
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) then RTD_BLOCK_HOUR + 1;
		else RTD_BLOCK_HOUR = .;

		/*Calculate Profit over Previous 2 Hours*/
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) and RTD_BLOCK_STARTUP_RECOVERED_DAM ~= 1 then do;
			if missing(_LAG1) then TWO_HR_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
			else TWO_HR_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT + _LAG1;
		end;

		/*See if Startup Costs Recovered and flag first hour recovered*/
		/*We subtract 1 for the two-hour case because we're measuring at the second hour.*/
		if ("&tech." = "F" or "&tech." = "H") and TWO_HR_PROFIT > RTD_STARTUP_COST then RTD_START_HOUR_FLAG = max(RTD_BLOCK_HOUR - 1, 1);
		else if ("&tech." = "LMS" or "&tech." = "W") and RTD_POTENTIAL_ENERGY_PROFIT > RTD_STARTUP_COST and RTD_BLOCK_STARTUP_RECOVERED_DAM ~= 1 then RTD_START_HOUR_FLAG = RTD_BLOCK_HOUR;

		label RTD_BLOCK_HOUR = 'Count Block Hours in Sequence, Used for Real-Time Amortization Decision';
		label TWO_HR_PROFIT = 'Rolling Two-Hour Unit Profit, Used for Real-Time Amortization Decision';
		label RTD_START_HOUR_FLAG = 'Temporary Variable, Used to Flag When Startup Costs Are Recovered';
	run;

	/*Flag all hours after start is triggered for energy dispatch*/
	proc sql;
		create table hr_profits_&tech.7 (drop = RTD_START_HOUR_FLAG) as select
			*,
			min(RTD_START_HOUR_FLAG) as RTD_START_HOUR label = 'The Block Hour in Which Unit Can Start and Be Assured Amortized Cost Recovery'
		from hr_profits_&tech.6
		group by AREA, RTD_POTENTIAL_BLOCK_NUMBER
		order by AREA, DATE, TIME;
	quit;

	/*    E. Update real time commitment to reflect flagged blocks*/
	/*1. If extending a DAM block, no additional startup cost [RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and DAM_COMMIT ~= "ENERGY"]*/
	/*2. If starting up "fresh", needs additional startup cost */

	data hr_profits_&tech.8 (drop = _: RTD_BLOCK_STARTUP_RECOVERED_DAM RTD_BLOCK_HOUR TWO_HR_PROFIT RTD_START_HOUR);
		set hr_profits_&tech.7;
		by AREA;
		retain _COUNTER _LAG1;
		if first.AREA then _COUNTER = 0;

		/*Flag Start Hour for Both Types of Real-Time Block (day-ahead extensions and new blocks)*/
		_LAG1 = lag1(RTD_BLOCK_STARTUP_RECOVERED_DAM);
		if (RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and _LAG1 = .) or (~missing(RTD_START_HOUR) and RTD_START_HOUR = RTD_BLOCK_HOUR) then _COUNTER + 1;

		/*1. If extending a DAM committed energy block, there is no additional startup cost to apply*/
		if RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and DAM_COMMIT ~= "ENERGY" then do;
			/*Reserve -> Energy. If DAM Reserve, unit also collects DAM Reserve price*/
			if DAM_COMMIT = "RESERVE" then do;
				RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT + DAM_HR_RESERVE_PROFIT;
				RTD_FINAL_HRLY_OTHER = DAM_HR_RESERVE_PROFIT - RTD_HR_RESERVE_PRICE;
				/*Account for the reserve-> energy day-ahead fuel credit*/
				RTD_FINAL_HRLY_PHYSICAL = RTD_HR_ENERGY_PROFIT + E_BUYOUT_FUEL_LOSS_COST;
			end;
			/*None -> Energy*/
			else do;
				RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = RTD_HR_ENERGY_PROFIT;
			end;

			RTD_DISPATCH = "ENERGY";
			FIRED_ENERGY_FUEL = ECONOMIC_FUEL;
		end;
		/*2. Otherwise, we need to apply a startup cost in the first hour*/
		else if (~missing(RTD_START_HOUR) and RTD_START_HOUR <= RTD_BLOCK_HOUR) then do;
			/*Apply startup in first hour*/
			if RTD_BLOCK_HOUR = RTD_START_HOUR then do;
				/*Reserve -> Energy*/
				if DAM_COMMIT = "RESERVE" then do;
					RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT - RTD_STARTUP_COST + DAM_HR_RESERVE_PROFIT;
					RTD_FINAL_HRLY_OTHER = DAM_HR_RESERVE_PROFIT - RTD_HR_RESERVE_PRICE;
					/*Account for the reserve-> energy day-ahead fuel credit*/
					RTD_FINAL_HRLY_PHYSICAL = (RTD_HR_ENERGY_PROFIT + E_BUYOUT_FUEL_LOSS_COST) - RTD_STARTUP_COST;
				end;
				/*None -> Energy*/
				else do;
					RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT - RTD_STARTUP_COST;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_HR_ENERGY_PROFIT - RTD_STARTUP_COST;
				end;
			end;
			/*All other hours*/
			else do;
				/*Reserve -> Energy*/
				if DAM_COMMIT = "RESERVE" then do;
					RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT + DAM_HR_RESERVE_PROFIT;
					RTD_FINAL_HRLY_OTHER = DAM_HR_RESERVE_PROFIT - RTD_HR_RESERVE_PRICE;
					/*Account for the reserve-> energy day-ahead fuel credit*/
					RTD_FINAL_HRLY_PHYSICAL = (RTD_HR_ENERGY_PROFIT + E_BUYOUT_FUEL_LOSS_COST);
				end;
				/*None -> Energy*/
				else do; 
					RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_POTENTIAL_ENERGY_PROFIT;
				end;
			end;

			RTD_DISPATCH = "ENERGY";
			FIRED_ENERGY_FUEL = ECONOMIC_FUEL;
		end;

		/*Update Block Number*/
		if RTD_DISPATCH = "ENERGY" then RTD_BLOCK_NUMBER = _COUNTER;
		label RTD_BLOCK_NUMBER = 'Real-Time Run Block Number';
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 8. RTD Non-Energy Hours - Determine if Units Buyout or Dispatch Real-Time Reserves 	  	   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*Rule 1. A unit switches from None to Reserve if positive profit (e.g., if the reserve PROFIT > 0)*/
	/*Rule 2. A unit switches from Reserve to None if the real-time reserve PRICE is 0*/
	data hr_profits_&tech.9;
		set hr_profits_&tech.8;

		/*Only act on hours not committed energy*/
		if missing(RTD_DISPATCH) then do;
			/*No DAM commitment*/
			/*Must cover cost of securing then not using fuel to commit reserves from no prior commitment*/
			if DAM_COMMIT = "NONE" and (RTD_HR_RESERVE_PRICE - E_BUYOUT_FUEL_LOSS_COST) > 0 then do;
				RTD_FINAL_HRLY_PROFIT = RTD_HR_RESERVE_PRICE - E_BUYOUT_FUEL_LOSS_COST;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = RTD_HR_RESERVE_PRICE - E_BUYOUT_FUEL_LOSS_COST;
				RTD_DISPATCH = "RESERVE";
			end;
			else if DAM_COMMIT = "NONE" then do;
				RTD_FINAL_HRLY_PROFIT = 0;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = 0;
				RTD_DISPATCH = "NONE";
			end;
			/*Reserve DAM commitment*/
			if DAM_COMMIT = "RESERVE" and RTD_HR_RESERVE_PRICE <= 0 then do;
				/*Reserve costs already accounted for in DAM_HR_RESERVE_PROFIT*/
				RTD_FINAL_HRLY_PROFIT = DAM_HR_RESERVE_PROFIT - RTD_HR_RESERVE_PRICE;
				RTD_FINAL_HRLY_OTHER = DAM_HR_RESERVE_PROFIT - RTD_HR_RESERVE_PRICE;
				RTD_FINAL_HRLY_PHYSICAL = 0;
				RTD_DISPATCH = "BUYOUT";
			end;
			else if DAM_COMMIT = "RESERVE" then do;
				/*Reserve costs already accounted for in DAM_HR_RESERVE_PROFIT*/
				RTD_FINAL_HRLY_PROFIT = DAM_HR_RESERVE_PROFIT;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = DAM_HR_RESERVE_PROFIT;
				RTD_FINAL_HRLY_PHYSICAL = DAM_HR_RESERVE_PROFIT;
				RTD_DISPATCH = "RESERVE";
			end;
		end;
	run;


	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 9. EFORd Adjustment  												   						   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	/*The EFORd adjustment impacts hours when the unit is committed to physically provide energy or 
		reserve services. It does not impact other profits due to buyouts, etc.*/

	data hr_profits_&tech.10;
		set hr_profits_&tech.9;
		EFORd_PROFIT_LOSS = (RTD_FINAL_HRLY_PHYSICAL * (1-EFORd_&tech.));
		RTD_FINAL_HRLY_PROFIT = RTD_FINAL_HRLY_PROFIT - (RTD_FINAL_HRLY_PHYSICAL * (1-EFORd_&tech.));
		label EFORd_PROFIT_LOSS = 'Reduction in Profit due to Unit EFORd ($/MWh)';
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 10. Run-Time Limit for NSPS 	  	   												   		   ;
	/*A: Calculate the runtime "Overage" (time over the limit) and keep only zones that must be limited*/
	/*B: Collapse by block and calculate the profit that would be lost by switching to DAM reserves*/
	/*C: Flag the least profitable blocks to turn off*/
	/*D: Merge turn off flag onto main dataset and limit energy runtime*/
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*A: Calculate the runtime "Overage" (time over the limit) and keep only zones that must be limited*/
	proc sql;
		create table runtime_&tech. (where = (RTD_DISPATCH = "ENERGY" and MODEL_YEAR_RUN_OVERAGE_&tech. > 0)) as select
			MODEL_YEAR, ZONE, AREA, RTD_BLOCK_NUMBER,
			count(case when RTD_DISPATCH = "ENERGY" then MODEL_YEAR else . end) as MODEL_YEAR_RUN_HOURS_&tech.,
			count(MODEL_YEAR) as MODEL_YEAR_TOTAL_HOURS_&tech.,
			&&&rt_limit_&tech. * calculated MODEL_YEAR_TOTAL_HOURS_&tech. as MODEL_YEAR_RUN_LIMIT_&tech.,
			max(0, calculated MODEL_YEAR_RUN_HOURS_&tech. - calculated MODEL_YEAR_RUN_LIMIT_&tech.) as MODEL_YEAR_RUN_OVERAGE_&tech.,
			RTD_DISPATCH,
			DAM_HR_RESERVE_PROFIT, RTD_FINAL_HRLY_PROFIT
		from hr_profits_&tech.10
		group by MODEL_YEAR, ZONE, AREA
		order by MODEL_YEAR, ZONE, AREA, RTD_BLOCK_NUMBER;
	quit;

	/*B: Collapse by block and calculate the profit that would be lost by switching to DAM reserves*/
	proc sql;
		create table runtime_&tech.2 as select
			MODEL_YEAR, ZONE, AREA, RTD_BLOCK_NUMBER,
			MODEL_YEAR_RUN_OVERAGE_&tech.,
			count(DAM_HR_RESERVE_PROFIT) as BLOCK_HOURS, 
			sum(max(DAM_HR_RESERVE_PROFIT,0)) as BLOCK_RESERVE_PROFITS, 
			sum(RTD_FINAL_HRLY_PROFIT) as BLOCK_RUN_PROFITS,
			calculated BLOCK_RUN_PROFITS - calculated BLOCK_RESERVE_PROFITS as BLOCK_FOREGONE_PROFIT
		from runtime_&tech.
		group by MODEL_YEAR, ZONE, AREA, RTD_BLOCK_NUMBER, MODEL_YEAR_RUN_OVERAGE_&tech.
		order by MODEL_YEAR, ZONE, AREA, BLOCK_FOREGONE_PROFIT;
	quit;

	/*C: Flag the least profitable blocks to turn off*/
	data runtime_&tech.3;
		set runtime_&tech.2;
		retain RUN_OVERAGE_REMAINING FOREGONE_PROFITS;
		by MODEL_YEAR ZONE AREA;

		if first.AREA then do;
			RUN_OVERAGE_REMAINING = MODEL_YEAR_RUN_OVERAGE_&tech.;
			FOREGONE_PROFITS = 0;
		end;

		if RUN_OVERAGE_REMAINING > 0 then do;
			TURN_OFF_FLAG = 1;
			RUN_OVERAGE_REMAINING = RUN_OVERAGE_REMAINING - BLOCK_HOURS;
			FOREGONE_PROFITS = FOREGONE_PROFITS + BLOCK_FOREGONE_PROFIT;
		end;
	run;

	/*D: Merge turn off flag onto main dataset and limit energy runtime*/
	proc sql;
		create table hr_profits_&tech.11 as select
	    	a.*,
			b.TURN_OFF_FLAG label = 'Flag to indicate unit will be prevented from running due to NSPS run hour limit'
		from hr_profits_&tech.10 as a
			left join runtime_&tech.3 as b
			on a.MODEL_YEAR = b.MODEL_YEAR and a.AREA = b.AREA and a.RTD_BLOCK_NUMBER = b.RTD_BLOCK_NUMBER;
	quit;
	data hr_profits_&tech.12;
		set hr_profits_&tech.11;
		if TURN_OFF_FLAG = 1 then do;
			RTD_FINAL_HRLY_PROFIT = max(DAM_HR_RESERVE_PROFIT, 0);
			RTD_DISPATCH = "LIMITED";
		end;
	run;

	/*Save results*/
	data neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;
		set hr_profits_&tech.12;
		proc sort; by AREA DATE TIME;
	run;
%mend eas_profits;
%eas_profits(tech=LMS);
%eas_profits(tech=F);
%eas_profits(tech=W);
%eas_profits(tech=H);

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 11. Summarize Model Results 	  	   														   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
%macro output_diagnostics(tech=);
	/*Adjust "tech" for the Wartsila Gas Only Case, which requires different unit parameters*/
	%if &tech. = W and &enable_gas_only. = yes %then %do;
		%put Tech was &tech. in gas only mode;
		%let tech = WG;
		%put Tech updated to &tech.;
	%end;

	/*Final Net EAS 3-Year Average*/
	data flag_starts_&tech.;
		set neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;
		if RTD_DISPATCH = "ENERGY" and missing(lag(RTD_BLOCK_NUMBER)) then UNIT_START = 1;
	run;

	proc sql;
		/*Sum to Monthly Level*/ 
		create table final_dispatch_mo_&tech. as select distinct
			AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, month(DATE) as MONTH,
			sum(RTD_FINAL_HRLY_PROFIT) as FINAL_HRLY_PROFIT_&tech.,
			sum(case when RTD_DISPATCH = "ENERGY" then 1 else 0 end) as FINAL_HRLY_COUNT_&tech.,
			sum(UNIT_START) as FINAL_START_COUNT_&tech.
		from flag_starts_&tech.
		group by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, MONTH
		order by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, MONTH;

		/*Average Each Month (over 3-years)*/
		create table final_dispatch_mo_&tech.2 as select distinct
			AREA, ZONE, CAPACITY_ZONE, MONTH,
			mean(FINAL_HRLY_PROFIT_&tech.) as FINAL_HRLY_PROFIT_&tech.,
			mean(FINAL_HRLY_COUNT_&tech.) as FINAL_HRLY_COUNT_&tech.,
			mean(FINAL_START_COUNT_&tech.) as FINAL_START_COUNT_&tech.
		from final_dispatch_mo_&tech.
		group by AREA, ZONE, CAPACITY_ZONE, MONTH
		order by AREA, ZONE, CAPACITY_ZONE, MONTH;

		/*Sum to Final*/
		create table final_dispatch_mo_&tech.3 as select distinct
			catx("_",AREA,ZONE) as LOOKUP,
			AREA, ZONE, CAPACITY_ZONE,
			sum(FINAL_HRLY_PROFIT_&tech.)/1000 + &vss_revenue. as FINAL_HRLY_PROFIT_&tech.,
			sum(FINAL_HRLY_COUNT_&tech.) as FINAL_HRLY_COUNT_&tech.,
			sum(FINAL_START_COUNT_&tech.) as FINAL_START_COUNT_&tech.
		from final_dispatch_mo_&tech.2
		group by AREA, ZONE, CAPACITY_ZONE
		order by AREA, ZONE, CAPACITY_ZONE;
	quit;

	/*Overview*/
	proc summary data = neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;
		class MODEL_YEAR AREA DAM_COMMIT RTD_DISPATCH;
		output out = summary_area_year_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_area_year_&tech.;
		format LOOKUP $100.;
		set summary_area_year_&tech.;
		LOOKUP = catx("-",MODEL_YEAR,AREA,DAM_COMMIT,RTD_DISPATCH);
	run;

	/*Overview - By Month*/
	data summary_area_month_&tech.;
		set neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;
		YEAR = year(DATE);
		MONTH = month(DATE);
	run;
	proc summary data = summary_area_month_&tech.;
		class YEAR MONTH AREA DAM_COMMIT RTD_DISPATCH;
		output out = summary_area_month_&tech.2 sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_area_month_&tech.;
		format LOOKUP $100.;
		set summary_area_month_&tech.2;
		LOOKUP = catx("-",YEAR,MONTH,AREA,DAM_COMMIT,RTD_DISPATCH);
	run;

	/*Oil/Gas*/
	proc summary data = neteas.modelout_&tech._&enable_gas_only._&enable_LOE. (where = (RTD_DISPATCH = "ENERGY"));
		class MODEL_YEAR AREA FIRED_ENERGY_FUEL;
		output out = summary_oil_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_oil_&tech.;
		format LOOKUP $100.;
		set summary_oil_&tech.;
		LOOKUP = catx("-",MODEL_YEAR,AREA,FIRED_ENERGY_FUEL);
	run;

	/*Summarize by Energy/Ancillary*/
	proc summary nway data = neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;
		class RTD_DISPATCH AREA MODEL_YEAR;
		output out = summ_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;

	/*Summarize by Year*/
	proc sql;
		create table yearly_summ_&tech. as select distinct
			catx("-",MODEL_YEAR,AREA) as LOOKUP,
			AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR,
			sum(RTD_FINAL_HRLY_PROFIT)/1000 + &vss_revenue. as FINAL_HRLY_PROFIT_&tech.,
			sum(case when RTD_DISPATCH = "ENERGY" then 1 else 0 end) as FINAL_HRLY_COUNT_&tech.,
			count(distinct case when RTD_DISPATCH = "ENERGY" then RTD_BLOCK_NUMBER else . end) as FINAL_START_COUNT_&tech.,
			mean((DAM_LBMP * LOE) - DAM_GAS_COST) as SPARK_SPREAD_&tech.
		from neteas.modelout_&tech._&enable_gas_only._&enable_LOE.
		group by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR
		order by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR;
	quit;

/*	data hourly_data_&tech.;*/
/*		retain  DATE TIME Zone AREA LOE DAM_LBMP RTD_LBMP DAM_30MIN DAM_10MIN_NONSPIN RTD_30MIN RTD_10MIN_NONSPIN DAM_GAS_PRICE RTD_GAS_PRICE_BUY RTD_GAS_PRICE_SELL ULSD_PRICE DAM_GAS_COST RTD_GAS_COST OIL_COST ECONOMIC_FUEL E_BUYOUT_FUEL_LOSS E_BUYOUT_FUEL_LOSS_COST DAM_GAS_SPARK_SPREAD DAM_HR_ENERGY_PROFIT RTD_HR_ENERGY_PROFIT DAM_STARTUP_COST RTD_STARTUP_COST E_BUYOUT_STARTUP_FUEL_LOSSES DAM_POTENTIAL_ENERGY_PROFIT DAM_POTENTIAL_BLOCK_NUMBER DAM_BLOCK_ENERGY_PROFIT DAM_BLOCK_NUMBER RTD_POTENTIAL_ENERGY_PROFIT RTD_POTENTIAL_BLOCK_NUMBER RTD_BLOCK_NUMBER DAM_COMMIT RTD_DISPATCH DAM_ENERGY DAM_RESERVE RTD_ENERGY RTD_RESERVE RTD_FINAL_HRLY_OTHER RTD_FINAL_HRLY_PHYSICAL EFORd_PROFIT_LOSS RTD_FINAL_HRLY_PROFIT;*/
/*		set neteas.modelout_&tech._&enable_gas_only._&enable_LOE.;*/
/*		keep DATE TIME Zone AREA LOE DAM_LBMP RTD_LBMP DAM_30MIN DAM_10MIN_NONSPIN RTD_30MIN RTD_10MIN_NONSPIN DAM_GAS_PRICE RTD_GAS_PRICE_BUY RTD_GAS_PRICE_SELL ULSD_PRICE DAM_GAS_COST RTD_GAS_COST OIL_COST ECONOMIC_FUEL E_BUYOUT_FUEL_LOSS E_BUYOUT_FUEL_LOSS_COST DAM_GAS_SPARK_SPREAD DAM_HR_ENERGY_PROFIT RTD_HR_ENERGY_PROFIT DAM_STARTUP_COST RTD_STARTUP_COST E_BUYOUT_STARTUP_FUEL_LOSSES DAM_POTENTIAL_ENERGY_PROFIT DAM_POTENTIAL_BLOCK_NUMBER DAM_BLOCK_ENERGY_PROFIT DAM_BLOCK_NUMBER RTD_POTENTIAL_ENERGY_PROFIT RTD_POTENTIAL_BLOCK_NUMBER RTD_BLOCK_NUMBER DAM_COMMIT RTD_DISPATCH DAM_ENERGY DAM_RESERVE RTD_ENERGY RTD_RESERVE RTD_FINAL_HRLY_OTHER RTD_FINAL_HRLY_PHYSICAL EFORd_PROFIT_LOSS RTD_FINAL_HRLY_PROFIT;*/
/**/
/*		if DAM_COMMIT = "ENERGY" then DAM_ENERGY = 1;*/
/*		if DAM_COMMIT = "RESERVE" then DAM_RESERVE = 1;*/
/*		if RTD_DISPATCH = "ENERGY" then RTD_ENERGY = 1;*/
/*		if RTD_DISPATCH = "RESERVE" then RTD_RESERVE = 1;*/
/*		DAM_GAS_SPARK_SPREAD = (DAM_LBMP * LOE) - DAM_GAS_COST;*/
/*	run;*/

	%macro export(data =, sheet =, file =);
		proc export data = &data. outfile = "&outpath.\&file..xlsx"
			dbms = XLSX replace;
			sheet = "&sheet.";
		run;
	%mend;
	
	%export(data=final_dispatch_mo_&tech.3, 	sheet=Final EAS &tech., 	file=&output_name.);
	%export(data=lk_summary_area_year_&tech., 	sheet=Results &tech., 		file=&output_name.);
	%export(data=lk_summary_area_month_&tech., 	sheet=Results M &tech., 		file=&output_name.);
	%export(data=lk_summary_oil_&tech., 		sheet=Oil Burn &tech., 		file=&output_name.);
	%export(data=summ_&tech., 					sheet=EAS Rev &tech., 		file=&output_name.);
	%export(data=yearly_summ_&tech., 			sheet=EAS Yr &tech., 		file=&output_name.);
/*	%export(data=hourly_data_&tech., 			sheet=&tech. Hourly Data, 	file=&output_name. Hourly);*/
%mend;
%output_diagnostics(tech=LMS);
%output_diagnostics(tech=F);
%output_diagnostics(tech=W);
%output_diagnostics(tech=H);
