***********************************************************************************************;
* DRAFT - CONFIDENTIAL		           									   		   			   ;
* Program:	Net EAS Revenues Model		           									   		   ;
* Date:		September 2020                                                     				   ;
* Author:	Analysis Group, Inc      														   ;
* Client: 	New York Independent System Operator											   ;
* Purpose:	Calculate 3-Year Average Net EAS Revenues for Peaking Unit						   ;  
***********************************************************************************************;

%let filepath =  /* Enter file path */;
%let output_name = EAS Model Output CC Dual Fuel;
/*Set to 'yes' for gas only*/
%let enable_gas_only = no;
%let enable_LOE = yes;

/*Default Modeling Period is September 2016 Through August 2019.*/

options mlogic mprint;
libname neteas "&filepath.\Datasets";
%let outpath = &filepath.\Output;
%let inpath = &filepath.\Input;

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 1. Load Annually Updated Parameters								   			   			   ;
* 2. Load DCR Set Parameters								   			   				   	   ;
* 3. Merge Together Price Data for Each Hour								   			   	   ;
* 4. Calculate Base DAM and RTD Energy Profits in Each Hour					   			   	   ;
* 5. DAM Energy Blocks - Set Day-Ahead Behavior and Determine if Real-Time Buyout Profitable   ;
* 6. DAM Non-Energy Hours - Determine if Units Commit Reserves or No Commitment 		       ;
* 7. RTD Energy Blocks - Determine if Profitable to Change Day-Ahead Non-Energy to 			   ;	
*						 Real-Time Energy Dispatch											   ;
* 8. RTD Non-Energy Hours - Determine if Units Buyout or Dispatch Real-Time Reserves 	  	   ;
* 9. Min Run Addition - Determine when CC Unit will Run at Min Load							   ;
* 10. EFORd Adjustment  	  	   												     		   ;
* 11. Output Model Results																	   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 1. Load Annually Updated Parameters								   			   			   ;
* 	A: Energy LBMP																			   ;	
* 	B: Nonspin Reserve Prices;
* 	C: Fuel Prices (Gas and Oil);
* 	D: Emission Prices (CO2, NOX, SO2);
* 	E: RS1;
* 	F: Modeling Dates;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*A: Energy LBMP*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = lbmp_dam 
	DBMS = XLSX replace;
	sheet = "DA LBMP - All Zones";
run;
data in_lbmp_dam (	drop = Eastern_Date_Hour Zone_PTID DAM_Zonal_Losses--DAM_Zonal_Price_Version
							rename = (DAM_Zonal_LBMP = DAM_LBMP Zone_Name = Zone)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set lbmp_dam;
	if Zone_Name = "N.Y.C." then Zone_Name = "NYC";
	else if Zone_Name = "HUD VL" then Zone_Name = "HUD_VL";
	DATE = datepart(Eastern_Date_Hour);
	TIME = timepart(Eastern_Date_Hour);
run;

/*RTD LMBP*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	out = lbmp_rtd
	DBMS = XLSX replace;
	sheet = "RT LBMP - All Zones";
run;
data in_lbmp_rtd (	drop = Eastern_Date_Hour Zone_PTID TWI_Zonal_Losses--TWI_Zonal_Price_Version
							rename = (TWI_Zonal_LBMP = RTD_LBMP Zone_Name = Zone)
							where = (~missing(DATE) and ~missing(TIME)));
	format DATE date10.;
	format TIME time8.;
	set lbmp_rtd;
	if Zone_Name = "N.Y.C." then Zone_Name = "NYC";
	else if Zone_Name = "HUD VL" then Zone_Name = "HUD_VL";
	DATE = datepart(Eastern_Date_Hour);
	TIME = timepart(Eastern_Date_Hour);
run;

/*B: Nonspin Reserve Prices*/
/*Not used in CC Model*/

/*Make adjustment for Daylight Savings Hour*/
%macro adjust_for_dls(data=);
	/*1. Split into duplicated (daylight savings) and non-duplicated hours*/
	proc sort nouniquekey data = &data. out = &data._dls uniqueout = &data._no_dls;
	      by DATE TIME ZONE;
	run;

	/*2. Recombine and increment 2nd daylight savings hour by 1 minute to make the timestamp unique. */
	/*	Model will process both records as distinct and full hours. */
	data &data._adj;
		set &data._dls (in = a) &data._no_dls (in = b);
		if a and mod(_N_,2) = 0 then TIME = TIME + '00:00:01't;
		proc sort; by DATE TIME ZONE;
	run;
%mend;
%adjust_for_dls(data = in_lbmp_dam);
%adjust_for_dls(data = in_lbmp_rtd);

/*C: Fuel Prices*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_f_gas (where = (~missing(DATE)));
	sheet = "SNL Gas";
run;
data in_f_gas_lag;
	set in_f_gas;
	/*DATE = DATE + 1;*/ /* 9.18.2020 Note: statement removed per new gas price methodology */
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_f_oil (where = (~missing(DAY)));
	sheet = "EIA Oil";
	proc sort nodupkey; by DAY;
run;

/*D: Emission Prices*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_co2 (rename = (Auction_Clearing_Price = CO2_PRICE) where = (~missing(AUCTION_DATE)));
	sheet = "RGGI CO2";
	proc sort nodupkey; by AUCTION_DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_nox (rename = (Seasonal_NOX = NOX_SEASONAL_PRICE Annual_NOX = NOX_ANNUAL_PRICE) where = (~missing(DATE)));
	sheet = "SNL NOx";
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_so2_CSAPR  (rename = (ALLOWANCE_PRICE = SO2_PRICE) where = (~missing(DATE)));
	sheet = "SNL CSAPR SO2";
	proc sort nodupkey; by DATE;
run;
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_e_so2_arp  (rename = (ALLOWANCE_PRICE = SO2_PRICE) where = (~missing(DATE)));
	sheet = "SNL ARP SO2";
	proc sort nodupkey; by DATE;
run;
/*Use EPA auction data prior to SNL coverage (Jan 1, 2015)*/
data in_e_so2_arp_epa;
	DATE = 2005; SO2_PRICE = 702.51; output;
	DATE = 2006; SO2_PRICE = 883.1; output;
	DATE = 2007; SO2_PRICE = 444.39; output;
	DATE = 2008; SO2_PRICE = 389.91; output;
	DATE = 2009; SO2_PRICE = 69.74; output;
	DATE = 2010; SO2_PRICE = 37.71; output;
	DATE = 2011; SO2_PRICE = 2.81; output;
	DATE = 2012; SO2_PRICE = 0.67; output;
	DATE = 2013; SO2_PRICE = 0.28; output;
	DATE = 2014; SO2_PRICE = 0.45; output;
run;

/*E: RS1*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_rs1 (where = (~missing(YEAR)));
	sheet = "NYISO RS1";
	proc sort nodupkey; by YEAR;
run;

/*F: Modeling Dates*/
proc import datafile = "&inpath.\Net EAS Model Inputs.xlsx"
	dbms = XLSX replace
	out = in_dates (where = (~missing(DATE)));
	sheet = "Dates";
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 2. Load DCR Set Parameters								   			   				   	   ;
/*	A: Fuel Transportation and Intraday*/
/*	B: Unit Characteristics */
/*  C: Modeling Decisions*/
/*  D: LOE Adjustment */
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*A: Fuel Transportation and Intraday*/
/*Natural Gas Transportation Costs (See IMM Table A-2)*/
	%let ros_fuel_trans = 0.27;
	%let hv_fuel_trans = 0.27;
	%let nyc_fuel_trans = 0.20;
	%let li_fuel_trans = 0.25;

/*Natural Gas Intraday Premium (See IMM Table A-2)*/
	%let ros_fuel_prem = 1.1;
	%let hv_fuel_prem = 1.1;
	%let nyc_fuel_prem = 1.2;
	%let li_fuel_prem = 1.3;
	%let ros_fuel_disc = 0.9;
	%let hv_fuel_disc = 0.9;
	%let nyc_fuel_disc = 0.8;
	%let li_fuel_disc = 0.7;

/*Gas excise taxes (See IMM Page A-23)*/
	%let nyc_gas_tax = 1.069;
	%let li_gas_tax = 1.01;
	
/*NYC 4.5% sales tax*/
	%let nyc_sales_tax = 1.045;

/*ULSD Transportation Costs (See IMM Table A-2)*/
	%let ros_ulsd_trans = 2;
	%let hv_ulsd_trans = 1.5;
	%let nyc_ulsd_trans = 1.5;
	%let li_ulsd_trans = 1.5;

/*B: Unit Characteristics (Provided by Lummus)*/
	/*Historical Average VSS and Ancillary revenues from NYISO data*/
	%let vss_revenue = 1.43;
	%let ancillary_revenue = 3.70;

	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_ccf;
		sheet = "tech_params_ccf";
	run;
	proc import datafile = "&inpath.\Lummus Performance and OM Data.xlsx"
		dbms = XLSX replace
		out = in_tech_cch;
		sheet = "tech_params_cch";
	run;

/*C: Modeling Decisions*/
	%let enable_day_ahead_buyout = yes;

/*D: LOE Adjustment*/
proc import datafile = "&inpath.\LOE AFs for Net EAS.xlsx"
	out = loe
	DBMS = XLSX replace;
	sheet = "LOE";
run;
data in_loe;
	format DATE date10.;
	format TIME time8.;
	set loe;
	TIME = hms(HOUR,0,SECOND);
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 3. Merge Together Price Data for Each Hour								   			   	   ;
	/*A: Merge Together Emissions Prices*/
	/*B: Merge Together Fuel Prices*/
	/*C: Format Energy LBMP and LOE*/
	/*D: Combine All Datasets*/
	/*E: Set Proper Gas Prices in Each Zone*/
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*Final format will be "Long" with one observation for each Zone-Hour*/

/*A: Merge Together Emissions Prices*/
/*Note: SO2 CSAPR includes SO CAIR Prior to 2015. The name is simply to distinguish from ARP allowances in this code*/
proc sql;
	create table prices_emissions as select
		coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE) as DATE format mmddyy10.,
    	a.CO2_PRICE,
		b.NOX_ANNUAL_PRICE,
		b.NOX_SEASONAL_PRICE,
		c.SO2_PRICE as SO2_CSAPR_PRICE,
		case when ~missing(e.SO2_PRICE) then e.SO2_PRICE else d.SO2_PRICE end as SO2_ARP_PRICE,
		dates.DATE as KEEP_DATES,
		dates.YEAR_NUMBER as MODEL_YEAR
	from in_dates as dates
		full outer join in_e_co2 as a
		on dates.DATE = a.AUCTION_DATE
		full outer join In_e_nox as b
		on coalesce(dates.DATE,a.AUCTION_DATE) = b.DATE
		full outer join In_e_so2_CSAPR as c
		on coalesce(dates.DATE,a.AUCTION_DATE,b.DATE) = c.DATE
		left join In_e_so2_arp_epa as d
		on year(coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE)) = d.DATE
		left join In_e_so2_arp as e
		on coalesce(dates.DATE,a.AUCTION_DATE,b.DATE,c.DATE) = e.DATE
	order by DATE;
quit;

/*Carry forward previous price when missing*/
data prices_emissions2 (drop = OLD: KEEP_DATES);
	retain DATE CO2_PRICE NOX_ANNUAL_PRICE NOX_SEASONAL_PRICE SO2_CSAPR_PRICE SO2_ARP_PRICE;
	set prices_emissions (rename = (CO2_PRICE=OLD_CO2_PRICE NOX_ANNUAL_PRICE=OLD_NOX_ANNUAL_PRICE NOX_SEASONAL_PRICE=OLD_NOX_SEASONAL_PRICE SO2_CSAPR_PRICE=OLD_SO2_CSAPR_PRICE SO2_ARP_PRICE=OLD_SO2_ARP_PRICE));
	if ~missing(OLD_CO2_PRICE) 			then CO2_PRICE = 			OLD_CO2_PRICE;
	if ~missing(OLD_NOX_ANNUAL_PRICE) 	then NOX_ANNUAL_PRICE = 	OLD_NOX_ANNUAL_PRICE;
	if ~missing(OLD_NOX_SEASONAL_PRICE) then NOX_SEASONAL_PRICE = 	OLD_NOX_SEASONAL_PRICE;
	if ~missing(OLD_SO2_CSAPR_PRICE) 	then SO2_CSAPR_PRICE= 		OLD_SO2_CSAPR_PRICE;
	if ~missing(OLD_SO2_ARP_PRICE) 		then SO2_ARP_PRICE= 		OLD_SO2_ARP_PRICE;
	
	/*Drop leading/lagging rows*/
	if ~missing(KEEP_DATES);
run;

/*B: Merge Together Fuel Prices*/
proc sql;
	create table prices_fuel as select 
		coalesce(dates.DATE, a.DATE, b.DAY) as DATE format mmddyy10.,
		a.Transco_Z6_Price,
		a.TGP_Z6_Price,
		a.TETCO_M3_Price,
		a.Iroquois_Z2_Price,
		a.Dominion_N_Price,
		a.Dominion_S_Price,
		a.Millennium_Price,
		/*Convert units from $/gallon to $/MMBtu using EIA conversion factor*/
		/*of 137,381 BTU/gallon												*/
		b.U_S__No_2_Diesel_Ultra_Low_Sulfu * 7.2790269396787 as ULSD_PRICE
	from in_dates as dates
		full outer join in_f_gas_lag as a
		on dates.DATE = a.DATE
		full outer join in_f_oil as b
		on coalesce(dates.DATE, a.DATE) = b.DAY
	order by DATE;
quit;

/*2020.09.21 Update: Carry backwards next fuel price when missing*/
proc sort data = prices_fuel;
      by descending DATE;
run;
data prices_fuel_fill (rename = (ULSD_PRICE_FILL = ULSD_PRICE DOMINION_N_PRICE_FILL = DOMINION_N_PRICE DOMINION_S_PRICE_FILL = DOMINION_S_PRICE IROQUOIS_Z2_PRICE_FILL = IROQUOIS_Z2_PRICE MILLENNIUM_PRICE_FILL = MILLENNIUM_PRICE TETCO_M3_PRICE_FILL = TETCO_M3_PRICE TGP_Z6_PRICE_FILL = TGP_Z6_PRICE TRANSCO_Z6_PRICE_FILL = TRANSCO_Z6_PRICE));
	set prices_fuel;
	retain DOMINION_N_PRICE_FILL DOMINION_S_PRICE_FILL IROQUOIS_Z2_PRICE_FILL 
		   MILLENNIUM_PRICE_FILL TETCO_M3_PRICE_FILL TGP_Z6_PRICE_FILL 
		   TRANSCO_Z6_PRICE_FILL ULSD_PRICE_FILL;
	by descending DATE;

	if ~missing(DOMINION_N_PRICE) then DOMINION_N_PRICE_FILL = DOMINION_N_PRICE;
	if ~missing(DOMINION_S_PRICE) then DOMINION_S_PRICE_FILL = DOMINION_S_PRICE;
	if ~missing(IROQUOIS_Z2_PRICE) then IROQUOIS_Z2_PRICE_FILL = IROQUOIS_Z2_PRICE;
	if ~missing(MILLENNIUM_PRICE) and MILLENNIUM_PRICE ~= "NA" then MILLENNIUM_PRICE_FILL = input(MILLENNIUM_PRICE, best.);
	if ~missing(TETCO_M3_PRICE) then TETCO_M3_PRICE_FILL = TETCO_M3_PRICE;
	if ~missing(TGP_Z6_PRICE) then TGP_Z6_PRICE_FILL = TGP_Z6_PRICE;
	if ~missing(TRANSCO_Z6_PRICE) then TRANSCO_Z6_PRICE_FILL = TRANSCO_Z6_PRICE;
	if ~missing(ULSD_PRICE) then ULSD_PRICE_FILL = ULSD_PRICE;

	drop DOMINION_N_PRICE DOMINION_S_PRICE IROQUOIS_Z2_PRICE MILLENNIUM_PRICE TETCO_M3_PRICE TGP_Z6_PRICE TRANSCO_Z6_PRICE ULSD_PRICE;
run;

proc sort data = prices_fuel_fill;
      by DATE;
run;


/*C: Format LOE*/
proc transpose data = in_loe
	out = in_loe_long (rename=(Col1=LOE)) name=Zone;
	by DATE TIME;
	var CAPITL CENTRL HUD_VL LONGIL NYC;
run;

/*D: Combine All Datasets*/
data merge_base;
	set in_lbmp_dam_adj;
	format AREA $2.;

	if zone = "CENTRL" then AREA = "C";
	else if zone = "CAPITL" then AREA = "F";
	else if zone = "HUD_VL" then AREA = "G";
	else if zone = "NYC" then AREA = "J";
	else if zone = "LONGIL" then AREA = "K";

	if AREA in ("C", "F") then CAPACITY_ZONE = "ROS";
	else if AREA in ("G") then CAPACITY_ZONE = "HV";
	else if AREA in ("J") then CAPACITY_ZONE = "NYC";
	else if AREA in ("K") then CAPACITY_ZONE = "LI";

	/*Duplicate Zone G for Rockland/Dutchess Permutations*/
	if zone = "HUD_VL" then do;
		AREA = "G1"; output;
		AREA = "G2"; output;
	end;
	else output;
run;

proc sql;
	create table merge_base2 as select
    	a.DATE label = 'Date', 
		a.TIME label = 'Time', 
		g.DATE as KEEP_DATES label = 'Flag for Dates in the Current Modeling Period',
		g.MODEL_YEAR label = 'Numeric Year in 3-Year Modeling Period (1, 2 or 3)',
		a.ZONE label = 'Zone Name', 
		a.AREA label = 'Zone Letter (A-K; G1 = Dutchess; G2 = Rockland)', 
		a.CAPACITY_ZONE label = 'Capacity Zone (LI, NYC, HV, ROS)',

		b.DAM_LBMP label = 'Day-Ahead Hourly Average LBMP',
		c.RTD_LBMP label = 'Real-Time Dispatch Hourly Average LBMP',

		loe.LOE label = 'Level of Excess Scaling Factor',

		f.TRANSCO_Z6_PRICE label = 'Transco Z6 NY Natural Gas Price ($/MMBtu)',
		f.TGP_Z6_PRICE label = 'TGP Z6 Natural Gas Price ($/MMBtu)',
		f.TETCO_M3_PRICE label = 'TETCO-M3 Natural Gas Price ($/MMBtu)',
		f.IROQUOIS_Z2_PRICE label = 'Iroquois Z2 Natural Gas Price ($/MMBtu)',
		f.DOMINION_N_PRICE label = 'Dominion North Natural Gas Price ($/MMBtu)',
		f.DOMINION_S_PRICE label = 'Dominion South Natural Gas Price ($/MMBtu)',
		f.ULSD_PRICE label = 'ULSD Oil Price ($/MMBtu)',

		g.CO2_PRICE label = 'RGGI CO2 Allowance Price ($/ton)',
		g.NOX_ANNUAL_PRICE label = 'CAIR/CSAPR NOX Annual Allowance Price ($/allowance)',
		g.NOX_SEASONAL_PRICE label = 'CAIR/CSAPR NOX Seasonal Allowance Price ($/allowance)',
		g.SO2_CSAPR_PRICE label = 'CAIR/CSAPR SO2 Allowance Price ($/allowance)',
		g.SO2_ARP_PRICE label = 'Acid Rain Program SO2 Allowance Price ($/allowance)',

		rs.RATE as RS1 label = 'Rate Schedule 1 Cost Adder ($/MWh)',

		/*Unit Parameters - CCF Machine*/
		case when month(a.DATE) in (5,6,7,8,9,10) then l.HR_SUM_CCF / 1000						else l.HR_WIN_CCF  / 1000	end as HR_CCF label = 'CC F Heat Rate (MMBtu/MWh)',
		l.VOC_NG_CCF label = 'CC F Gas-Fired Variable Operating Cost ($/MWh)',
		l.VOC_OIL_CCF label = 'CC F Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.ICAP_SUM_CCF 							else l.ICAP_WIN_CCF end as ICAP_CCF label = 'CC F ICAP (MW)',
		l.STARTUP_FUEL_CCF label = 'CC F Startup Fuel Burn (MMBtu)',
		l.STARTUP_VOM_CCF label = 'CC F Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.CO2_NG_SUM_CCF  / l.ICAP_SUM_CCF / 2000	else l.CO2_NG_WIN_CCF  / l.ICAP_WIN_CCF / 2000	end as CO2_NG_CCF label = 'CC F Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.CO2_OIL_SUM_CCF / l.ICAP_WIN_CCF / 2000	else l.CO2_OIL_WIN_CCF / l.ICAP_WIN_CCF / 2000	end as CO2_OIL_CCF label = 'CC F Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.NOX_NG_SUM_CCF  / l.ICAP_SUM_CCF / 2000	else l.NOX_NG_WIN_CCF  / l.ICAP_WIN_CCF / 2000	end as NOX_NG_CCF label = 'CC F Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.NOX_OIL_SUM_CCF / l.ICAP_WIN_CCF / 2000	else l.NOX_OIL_WIN_CCF / l.ICAP_WIN_CCF / 2000	end as NOX_OIL_CCF label = 'CC F Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.SO2_NG_SUM_CCF  / l.ICAP_SUM_CCF / 2000	else l.SO2_NG_WIN_CCF  / l.ICAP_WIN_CCF / 2000	end as SO2_NG_CCF label = 'CC F Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.SO2_OIL_SUM_CCF / l.ICAP_WIN_CCF / 2000	else l.SO2_OIL_WIN_CCF / l.ICAP_WIN_CCF / 2000	end as SO2_OIL_CCF label = 'CC F Oil-Fired SO2 Emissions (lb/hr)',
		l.EFORd_CCF label = 'CC F EFORd (%)',
		/*Min Run Parameters*/
		case when month(a.DATE) in (5,6,7,8,9,10) then l.HR_SUM_MIN_CCF / 1000						else l.HR_WIN_MIN_CCF  / 1000	end as HR_MIN_CCF label = 'CC F Heat Rate at Min Load (MMBtu/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then l.ICAP_SUM_MIN_CCF/l.ICAP_SUM_CCF 			else l.ICAP_WIN_MIN_CCF / l.ICAP_WIN_CCF end as MIN_ICAP_RATIO_CCF label = 'CC F Min Load ICAP Ratio',

		/*Unit Parameters - CCH Machine*/
		case when month(a.DATE) in (5,6,7,8,9,10) then m.HR_SUM_CCH / 1000						else m.HR_WIN_CCH  / 1000	end as HR_CCH label = 'CC H Heat Rate (MMBtu/MWh)',
		m.VOC_NG_CCH label = 'CC H Gas-Fired Variable Operating Cost ($/MWh)',
		m.VOC_OIL_CCH label = 'CC H Oil-Fired Variable Operating Cost ($/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.ICAP_SUM_CCH 							else m.ICAP_WIN_CCH end as ICAP_CCH label = 'CC H ICAP (MW)',
		m.STARTUP_FUEL_CCH label = 'CC H Startup Fuel Burn (MMBtu)',
		m.STARTUP_VOM_CCH label = 'CC H Startup Variable Cost ($/Start)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.CO2_NG_SUM_CCH  / m.ICAP_SUM_CCH / 2000	else m.CO2_NG_WIN_CCH  / m.ICAP_WIN_CCH / 2000	end as CO2_NG_CCH label = 'CC H Gas-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.CO2_OIL_SUM_CCH / m.ICAP_WIN_CCH / 2000	else m.CO2_OIL_WIN_CCH / m.ICAP_WIN_CCH / 2000	end as CO2_OIL_CCH label = 'CC H Oil-Fired CO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.NOX_NG_SUM_CCH  / m.ICAP_SUM_CCH / 2000	else m.NOX_NG_WIN_CCH  / m.ICAP_WIN_CCH / 2000	end as NOX_NG_CCH label = 'CC H Gas-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.NOX_OIL_SUM_CCH / m.ICAP_WIN_CCH / 2000	else m.NOX_OIL_WIN_CCH / m.ICAP_WIN_CCH / 2000	end as NOX_OIL_CCH label = 'CC H Oil-Fired NOX Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.SO2_NG_SUM_CCH  / m.ICAP_SUM_CCH / 2000	else m.SO2_NG_WIN_CCH  / m.ICAP_WIN_CCH / 2000	end as SO2_NG_CCH label = 'CC H Gas-Fired SO2 Emissions (lb/hr)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.SO2_OIL_SUM_CCH / m.ICAP_WIN_CCH / 2000	else m.SO2_OIL_WIN_CCH / m.ICAP_WIN_CCH / 2000	end as SO2_OIL_CCH label = 'CC H Oil-Fired SO2 Emissions (lb/hr)',
		m.EFORd_CCH label = 'CC H EFORd (%)',
		/*Min Run Parameters*/
		case when month(a.DATE) in (5,6,7,8,9,10) then m.HR_SUM_MIN_CCH / 1000						else m.HR_WIN_MIN_CCH  / 1000	end as HR_MIN_CCH label = 'CC H Heat Rate at Min Load (MMBtu/MWh)',
		case when month(a.DATE) in (5,6,7,8,9,10) then m.ICAP_SUM_MIN_CCH/m.ICAP_SUM_CCH 			else m.ICAP_WIN_MIN_CCH / m.ICAP_WIN_CCH end as MIN_ICAP_RATIO_CCH label = 'CC H Min Load ICAP Ratio'

	from merge_base as a
		left join In_lbmp_dam_adj as b
			on a.DATE = b.DATE and a.TIME = b.TIME and a.ZONE = b.ZONE
		left join In_lbmp_rtd_adj as c
			on a.DATE = c.DATE and a.TIME = c.TIME and a.ZONE = c.ZONE
		left join Prices_fuel_fill as f
			on a.DATE = f.DATE
		left join Prices_emissions2 as g
			on a.DATE = g.DATE
		left join in_rs1 as rs
			on year(a.DATE) = rs.YEAR
		left join IN_LOE_LONG as loe
			on a.DATE = loe.DATE and a.TIME = loe.TIME and a.ZONE = loe.ZONE
		inner join in_tech_ccf as l
			on a.AREA = l.AREA
		inner join in_tech_cch as m
			on a.AREA = m.AREA
	order by AREA, DATE, TIME;
quit;

/*E: Set Proper Gas in Each Zone*/
data merge_data;
	set merge_base2;

	/*Assign Gas Prices to each Area*/
	if AREA in ("C") and ~missing(TETCO_M3_PRICE) then do;
		DAM_GAS_PRICE = TETCO_M3_PRICE + &ros_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (TETCO_M3_PRICE*&ros_fuel_prem.) + &ros_fuel_trans.;
		RTD_GAS_PRICE_SELL = (TETCO_M3_PRICE*&ros_fuel_disc.) + &ros_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &ros_ulsd_trans.;
	end;
	if AREA in ("F") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &ros_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&ros_fuel_prem.) + &ros_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&ros_fuel_disc.) + &ros_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &ros_ulsd_trans.;
	end;
	else if AREA in ("G2") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &hv_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&hv_fuel_prem.) + &hv_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&hv_fuel_disc.) + &hv_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &hv_ulsd_trans.;
	end;
	else if AREA in ("G1") and ~missing(IROQUOIS_Z2_PRICE) then do;
		DAM_GAS_PRICE = IROQUOIS_Z2_PRICE + &hv_fuel_trans.;
		RTD_GAS_PRICE_BUY  = (IROQUOIS_Z2_PRICE*&hv_fuel_prem.) + &hv_fuel_trans.;
		RTD_GAS_PRICE_SELL = (IROQUOIS_Z2_PRICE*&hv_fuel_disc.) + &hv_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &hv_ulsd_trans.;
	end;
	else if AREA in ("J") and ~missing(TRANSCO_Z6_PRICE) then do;
		DAM_GAS_PRICE = (TRANSCO_Z6_PRICE*&nyc_gas_tax.) + &nyc_fuel_trans.;
		RTD_GAS_PRICE_BUY  = ((TRANSCO_Z6_PRICE*&nyc_fuel_prem.)*&nyc_gas_tax.) + &nyc_fuel_trans.;
		RTD_GAS_PRICE_SELL = ((TRANSCO_Z6_PRICE*&nyc_fuel_disc.)*&nyc_gas_tax.) + &nyc_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = (ULSD_PRICE*&nyc_sales_tax.) + &nyc_ulsd_trans.;
	end;
	else if AREA in ("K") and ~missing(TRANSCO_Z6_PRICE) then do;
		DAM_GAS_PRICE = (TRANSCO_Z6_PRICE*&li_gas_tax.) + &li_fuel_trans.;
		RTD_GAS_PRICE_BUY  = ((TRANSCO_Z6_PRICE*&li_fuel_prem.)*&li_gas_tax.) + &li_fuel_trans.;
		RTD_GAS_PRICE_SELL = ((TRANSCO_Z6_PRICE*&li_fuel_disc.)*&li_gas_tax.) + &li_fuel_trans.;
		if ~missing(ULSD_PRICE) then ULSD_PRICE = ULSD_PRICE + &li_ulsd_trans.;
	end;

run;

/*Carry forward fuel prices when missing*/
proc sort data = merge_data;
      by AREA DATE TIME;
run;

data netEAS.full_input_data_cc (drop = KEEP_DATES);
	set merge_data;

	if missing(CO2_PRICE) then CO2_PRICE = 0;
	if missing(SO2_CSAPR_PRICE) then SO2_CSAPR_PRICE = 0;
	if missing(SO2_ARP_PRICE) then SO2_ARP_PRICE = 0;
	if missing(NOX_ANNUAL_PRICE) then NOX_ANNUAL_PRICE = 0;
	if missing(NOX_SEASONAL_PRICE) then NOX_SEASONAL_PRICE = 0;
	if missing(RS1) then RS1 = 0;
	if missing(LOE) then LOE = 1;

	/*During NOX Season, plant owners must purchase both Seasonal and Annual Credits*/
	if month(DATE) in (5,6,7,8,9) then NOX_PRICE = NOX_ANNUAL_PRICE+NOX_SEASONAL_PRICE;
	else NOX_PRICE = NOX_ANNUAL_PRICE;

	/*Units must comply with both SO2 CAIR/CSARR and ARP Regulations*/
	/*From 2010-2014, CAIR allowances only covered half a ton of emissions, effectively doubling the price to cover one ton of emissions*/
	if year(DATE) in (2010,2011,2012,2013,2014) then SO2_PRICE = (2*SO2_CSAPR_PRICE)+SO2_ARP_PRICE;
	else SO2_PRICE = (1*SO2_CSAPR_PRICE)+SO2_ARP_PRICE;

	if ~missing(KEEP_DATES);

	label DAM_GAS_PRICE = 'Final Day-Ahead Natural Gas Price ($/MMBtu)';
	label RTD_GAS_PRICE_BUY = 'Final Real-Time Dispatch Natural Gas Price For Unit Purchase with Premium ($/MMBtu)';
	label RTD_GAS_PRICE_SELL = 'Final Real-Time Dispatch Natural Gas Price For Unit Buyout at Discount ($/MMBtu)';
	label ULSD_PRICE =  'ULSD Oil Price ($/MMBtu)';
	label NOX_PRICE = 'Total NOX Allowance Price ($/ton)';
	label SO2_PRICE = 'Total SO2 Allowance Price ($/ton)';
run;


*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 4. Calculate Base DAM and RTD Energy Profits in Each Hour					   			   	   ;
* 	 A: Decide to run on Oil vs. Gas;
*	 B: Calculate Potential Real-Time Fuel Penalty;
* 	 C: Determine Proper Reserve Type for Unit (10-min spinning or not);
*    D: Calculate Hourly Energy Profit and Total Cost Prior to Startup Costs by Zone;
*    E: Calculate Startup Costs if Unit Starts in Given Hour;
*    NOTE: All fuel calculations also calculate the 'loss' if a unit is to buy-out of the
		commitment later and not use the fuel												   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

/*Calculate potential hourly profit in Energy*/
%macro eas_profits(tech=);
	/*A: Decide to run on Oil vs. Gas*/
	data hr_profits_fuel_&tech.;
		set netEAS.full_input_data_cc;
		/*LOE Disable*/
		%if &enable_LOE. ~= yes %then %do; LOE = 1; %end;

		/*If gas only, set impossibly high oil price and drop NYC and LI*/
		%if &enable_gas_only. = yes %then %do; 
			ULSD_PRICE = 99999;
			if AREA in ('J','K') then delete; 
		%end;

		/*Gas*/
		DAM_GAS_FUEL_COST = (HR_&tech. * DAM_GAS_PRICE); label DAM_GAS_FUEL_COST = 'Day-Ahead Gas Cost ($/MWh)';
		RTD_GAS_FUEL_COST = (HR_&tech. * RTD_GAS_PRICE_BUY); label RTD_GAS_FUEL_COST = 'Real-Time Gas Cost ($/MWh)';
		GAS_EMISSIONS_COST = (CO2_NG_&tech.*CO2_PRICE) + (NOX_NG_&tech.*NOX_PRICE) + (SO2_NG_&tech.*SO2_PRICE); label GAS_EMISSIONS_COST = 'Gas Emissions Cost ($/MWh)';
		DAM_GAS_COST = DAM_GAS_FUEL_COST + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST; label DAM_GAS_COST = 'Total Day-Ahead Cost when Gas-Fired ($/MWh)';
		RTD_GAS_COST = RTD_GAS_FUEL_COST + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST; label RTD_GAS_COST = 'Total Real-Time Cost when Gas-Fired ($/MWh)';

		/*Min Run Cost - Assumme all costs vary linearly with the ICAP ratio, but at a higher heat rate*/
		DAM_GAS_FUEL_COST_MIN = (HR_MIN_&tech. * DAM_GAS_PRICE); label DAM_GAS_FUEL_COST_MIN = 'Day-Ahead Gas Cost at Min Load Heat Rate ($/MWh)';
		RTD_GAS_FUEL_COST_MIN = (HR_MIN_&tech. * RTD_GAS_PRICE_BUY); label RTD_GAS_FUEL_COST_MIN = 'Real-Time Gas Cost at Min Load Heat Rate ($/MWh)';
		DAM_GAS_COST_MIN = MIN_ICAP_RATIO_&tech. * (DAM_GAS_FUEL_COST_MIN + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST); label DAM_GAS_COST_MIN = 'Total Day-Ahead Cost when Gas-Fired at Min Load ($/Full Load MWh)';
		RTD_GAS_COST_MIN = MIN_ICAP_RATIO_&tech. * (RTD_GAS_FUEL_COST_MIN + VOC_NG_&tech. + RS1 + GAS_EMISSIONS_COST); label RTD_GAS_COST_MIN = 'Total Real-Time Cost when Gas-Fired at Min Load ($/Full Load MWh)';

		/*Oil*/
		OIL_FUEL_COST = (HR_&tech. * ULSD_PRICE); label OIL_FUEL_COST = 'Oil Cost ($/MWh)';
		OIL_EMISSIONS_COST = (CO2_OIL_&tech.*CO2_PRICE) + (NOX_OIL_&tech.*NOX_PRICE) + (SO2_OIL_&tech.*SO2_PRICE); label OIL_EMISSIONS_COST = 'Oil Emissions Cost ($/MWh)';
		OIL_COST = OIL_FUEL_COST + VOC_OIL_&tech. + RS1 + OIL_EMISSIONS_COST; label OIL_COST = 'Total Cost when Oil-Fired ($/MWh)';

		/*Min Run Cost - Assumme all costs vary linearly with the ICAP ratio, but at a higher heat rate*/
		OIL_FUEL_COST_MIN = (HR_MIN_&tech. * ULSD_PRICE); label OIL_FUEL_COST_MIN = 'Oil Cost at Min Load Heat Rate ($/MWh)';
		OIL_COST_MIN = MIN_ICAP_RATIO_&tech. * (OIL_FUEL_COST_MIN + VOC_OIL_&tech. + RS1 + OIL_EMISSIONS_COST); label OIL_COST_MIN = 'Total Cost when Oil-Fired at Min Load ($/Full Load MWh)';

		/*Select between fuels using day-ahead gas price*/
		if OIL_COST < DAM_GAS_COST and ~missing(ULSD_PRICE) then do;
			ECONOMIC_FUEL = "OIL";
			DAM_ECONOMIC_FUEL_PRICE = ULSD_PRICE;
			RTD_ECONOMIC_FUEL_PRICE = ULSD_PRICE;
			DAM_COST = OIL_COST;
			RTD_COST = OIL_COST;

			DAM_COST_MIN = OIL_COST_MIN;
			RTD_COST_MIN = OIL_COST_MIN;
		end; else do;
			ECONOMIC_FUEL = "GAS"; label ECONOMIC_FUEL = 'Day-Ahead Economic Fuel Type';
			DAM_ECONOMIC_FUEL_PRICE = DAM_GAS_PRICE; label DAM_ECONOMIC_FUEL_PRICE = 'Total Day-Ahead Fuel Price ($/MMBtu)';
			RTD_ECONOMIC_FUEL_PRICE = RTD_GAS_PRICE_BUY; label RTD_ECONOMIC_FUEL_PRICE = 'Total Real-Time Fuel Price ($/MMBtu)';
			DAM_COST = DAM_GAS_COST; label DAM_COST = 'Total Day-Ahead Cost ($/MWh)';
			RTD_COST = RTD_GAS_COST; label RTD_COST = 'Total Real-Time Cost ($/MWh)';

			DAM_COST_MIN = DAM_GAS_COST_MIN; label DAM_COST_MIN = 'Total Day-Ahead Cost at Min Load ($/Full Load MWh)';
			RTD_COST_MIN = RTD_GAS_COST_MIN; label RTD_COST_MIN = 'Total Real-Time Cost at Min Load ($/Full Load MWh)';
		end;

		/*Calculate fuel losses if unit buys-out of day-ahead fuel commitment*/
		if ECONOMIC_FUEL = "GAS" then E_BUYOUT_FUEL_LOSS = DAM_GAS_PRICE - RTD_GAS_PRICE_SELL;
		else if ECONOMIC_FUEL = "OIL" then E_BUYOUT_FUEL_LOSS = 0;
		label E_BUYOUT_FUEL_LOSS = '$ Loss per MMBtu from Day-Ahead Fuel Commitment Buyout ($/MMBtu)';
	run;

	/*Calculate Potential Energy Profits in Each Hour (and Startup Costs)*/
	data hr_profits_dam_&tech.;
		set hr_profits_fuel_&tech.;

	/*   B: Calculate Potential Real-Time Fuel Penalty*/
		E_BUYOUT_FUEL_LOSS_COST = (HR_&tech. * E_BUYOUT_FUEL_LOSS); label E_BUYOUT_FUEL_LOSS_COST = '$ Loss per MWh from Day-Ahead Fuel Commitment Buyout ($/MWh)';

	/* 	 C: Determine Proper Reserve Type for Unit (10-min spinning or not)*/
		/*Not used in CC Model*/

	/*    D: Calculate Hourly Energy Profit and Total Cost Prior to Startup Costs by Zone*/
		/*Hard cap at price of $4,000/MWh (transmission shortage cost maximum pricing)*/
		DAM_HR_ENERGY_PROFIT = (min(LOE*DAM_LBMP,4000) - DAM_COST); label DAM_HR_ENERGY_PROFIT = 'Day-Ahead Potential Energy Profit ($/MWh)';
		RTD_HR_ENERGY_PROFIT = (min(LOE*RTD_LBMP,4000) - RTD_COST); label RTD_HR_ENERGY_PROFIT = 'Real-Time Potential Energy Profit ($/MWh)';

		/*Min Run Cost - Assumme revenues vary linearly with the ICAP ratio*/
		DAM_HR_ENERGY_PROFIT_MIN = ((MIN_ICAP_RATIO_&tech. * min(LOE*DAM_LBMP,4000)) - DAM_COST_MIN); label DAM_HR_ENERGY_PROFIT_MIN = 'Day-Ahead Potential Energy Profit at Min Load ($/MWh)';
		RTD_HR_ENERGY_PROFIT_MIN = ((MIN_ICAP_RATIO_&tech. * min(LOE*RTD_LBMP,4000)) - RTD_COST_MIN); label RTD_HR_ENERGY_PROFIT_MIN = 'Real-Time Potential Energy Profit at Min Load ($/MWh)';

	/*    E: Calculate Startup Costs if Unit Starts in Given Hour*/
		DAM_STARTUP_FUEL_COST = (STARTUP_FUEL_&tech. * DAM_ECONOMIC_FUEL_PRICE) / ICAP_&tech.; label DAM_STARTUP_FUEL_COST = 'Day-Ahead Fuel Cost to Startup in Hour ($/MWh)';
		DAM_STARTUP_VOM_COST = (STARTUP_VOM_&tech.) / ICAP_&tech.; label DAM_STARTUP_VOM_COST = 'Day-Ahead Variable Cost to Startup in Hour ($/MWh)';
		DAM_STARTUP_COST = DAM_STARTUP_FUEL_COST + DAM_STARTUP_VOM_COST; label DAM_STARTUP_COST = 'Total Day-Ahead Cost to Startup in Hour ($/MWh)';

		RTD_STARTUP_FUEL_COST = (STARTUP_FUEL_&tech. * RTD_ECONOMIC_FUEL_PRICE) / ICAP_&tech.; label RTD_STARTUP_FUEL_COST = 'Real-Time Fuel Cost to Startup in Hour ($/MWh)';
		RTD_STARTUP_VOM_COST = (STARTUP_VOM_&tech.) / ICAP_&tech.; label RTD_STARTUP_VOM_COST = 'Real-Time Variable Cost to Startup in Hour ($/MWh)';
		RTD_STARTUP_COST = RTD_STARTUP_FUEL_COST + RTD_STARTUP_VOM_COST; label RTD_STARTUP_COST = 'Total Real-Time Cost to Startup in Hour ($/MWh)';

		E_BUYOUT_STARTUP_FUEL_LOSSES = (STARTUP_FUEL_&tech. * (E_BUYOUT_FUEL_LOSS)) / ICAP_&tech.; label E_BUYOUT_STARTUP_FUEL_LOSSES = '$ Startup Fuel Loss per MWh from Day-Ahead Energy Commitment Buyout ($/MWh)';
	run;

	proc sort data = hr_profits_dam_&tech.;
	      by AREA DATE TIME;
	run;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 5. DAM Energy Blocks - Set Day-Ahead Behavior and Determine if Real-Time Buyout Profitable   ;
	* 	 A: Flag "Blocks" of hours with positive hourly energy profit (pre startup cost);
	* 	 B: Ensure LMS unit can recover all costs within 1-hour amortization period;
	* 	 C: Sum profit and cost components for each block to use in block-level commitment logic;
	* 	 D: Decide if unit will commit energy in day-ahead
			1. Energy Block Profits (with Startup) > 0
	* 	 E: Decide if unit will stick to energy in real-time or buyout 
			NOTE: We assume a unit will not buy-out of a partial block to simplify calculations
			See logic as described in step D
	* 	 F: Merge block-level decision back on to hour-level data;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*	A: Flag "Blocks" of hours with positive hourly energy profit*/
	data blk_hrly_profits_dam_&tech. (drop = _:);
		set hr_profits_dam_&tech.;
		by AREA DATE TIME;
		retain _COUNTER;

		/*Select Profitable DAM Hours*/
		if DAM_HR_ENERGY_PROFIT > 0 then DAM_POTENTIAL_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT;

		/*Label DAM Run Blocks and Apply Start Costs*/
		/*Counter to Count Run Blocks*/
		if first.AREA then _COUNTER = 0;

		_LAG1 = lag(DAM_POTENTIAL_ENERGY_PROFIT);
		/*Account for Startup Cost in First Hour of Block*/
		if ~missing(DAM_POTENTIAL_ENERGY_PROFIT) and _LAG1 = . then do;
			_COUNTER + 1;
			DAM_POTENTIAL_ENERGY_PROFIT = DAM_POTENTIAL_ENERGY_PROFIT - DAM_STARTUP_COST;
		end;

		if ~missing(DAM_POTENTIAL_ENERGY_PROFIT) then DAM_POTENTIAL_BLOCK_NUMBER = _COUNTER;

		label DAM_POTENTIAL_ENERGY_PROFIT = 'Day-Ahead Potential Run Block Energy Profit ($/MWh)';
		label DAM_POTENTIAL_BLOCK_NUMBER = 'Day-Ahead Potential Run Block Number';
	run;

	/*	B: Ensure LMS and Wartsila units can recover all costs within 1-hour amortization period*/
	%macro DAM_amort();
	%if &tech. = LMS or &tech. = W %then %do;
		data blk_hrly_profits_dam_&tech.2 (drop = LAG_T _COUNTER TRIGGER_STARTUP);
			set blk_hrly_profits_dam_&tech.;
			by AREA DATE TIME;
			retain TRIGGER_STARTUP;

			if first.AREA then _COUNTER = 0;
			/*Flag that unit to run in the first hour where costs are recovered*/ 
			if DAM_HR_ENERGY_PROFIT <= 0 then TRIGGER_STARTUP = .;
			else if DAM_HR_ENERGY_PROFIT - DAM_STARTUP_COST > 0 then TRIGGER_STARTUP = 1;

			LAG_T = lag1(TRIGGER_STARTUP);
			/*Create variable*/ 
			if LAG_T = . and TRIGGER_STARTUP = 1 then do;
				DAM_BLOCK_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT - DAM_STARTUP_COST;
				_COUNTER + 1;
			end;
			else if TRIGGER_STARTUP = 1 then DAM_BLOCK_ENERGY_PROFIT = DAM_HR_ENERGY_PROFIT;

			/*Count Block #s*/
			if ~missing(TRIGGER_STARTUP) then DAM_BLOCK_NUMBER = _COUNTER;

			label DAM_BLOCK_ENERGY_PROFIT = 'Day-Ahead Run Block Energy Profit ($/MWh)';
			label DAM_BLOCK_NUMBER = 'Day-Ahead Run Block Number';
		run;
	%end;
	/*F-CLASS AND H MACHINE DO NOT NEED 1-HR AMORT*/
	%else %do;
		data blk_hrly_profits_dam_&tech.2;
			set blk_hrly_profits_dam_&tech.;
			DAM_BLOCK_NUMBER = DAM_POTENTIAL_BLOCK_NUMBER;
			DAM_BLOCK_ENERGY_PROFIT = DAM_POTENTIAL_ENERGY_PROFIT;

			label DAM_BLOCK_ENERGY_PROFIT = 'Day-Ahead Run Block Energy Profit ($/MWh)';
			label DAM_BLOCK_NUMBER = 'Day-Ahead Run Block Number';
		run;
	%end;
	%mend;
	%DAM_amort;

	/* 	 C: Sum profit and cost components for each block to use in block-level commitment logic*/
	proc sql;
		create table blk_profits_dam_&tech. as select distinct
			AREA, DAM_BLOCK_NUMBER,
			count(DAM_BLOCK_NUMBER) as DAM_BLOCK_HOURS label = 'Hours in Day-Ahead Block',
			sum(DAM_BLOCK_ENERGY_PROFIT) as DAM_BLOCK_ENERGY_PROFIT label = 'Day-Ahead Block Energy Profit ($)',
			sum(LOE*DAM_LBMP) as BLOCK_DAM_LBMP label = 'Day-Ahead Block LBMP ($)',
			sum(LOE*RTD_LBMP) as BLOCK_RTD_LBMP label = 'Real-Time Block LBMP ($)',
			sum(DAM_COST) + case when DATE = min(DATE) then DAM_STARTUP_COST else . end as BLOCK_COSTS label = 'Day-Ahead Block Costs ($)',
			sum(E_BUYOUT_FUEL_LOSS_COST) + max(E_BUYOUT_STARTUP_FUEL_LOSSES) as BUYOUT_FUEL_LOSSES label = '$ Loss per Block from Day-Ahead Energy Fuel Commitment Buyout ($)'
		from blk_hrly_profits_dam_&tech.2
		where ~missing(DAM_BLOCK_NUMBER)
		group by AREA, DAM_BLOCK_NUMBER
		having ~missing(BLOCK_COSTS)
		order by AREA, DAM_BLOCK_NUMBER;
	quit;

	/* 	 D: Decide if unit will commit energy in day-ahead
			1. Energy Block Profits (with Startup) > 0*/
	data blk_profits_dam_&tech.2;
		set blk_profits_dam_&tech.;

		/*Decide if DAM will Commit Energy*/
		format DAM_COMMIT $7.;
		if DAM_BLOCK_ENERGY_PROFIT > 0 then do;
			DAM_COMMIT = "ENERGY";
		end;
		label DAM_COMMIT = 'Day-Ahead Commitment (Energy/None)';
	run;

	* 	 E: Decide if unit will stick to energy in real-time or buyout
			NOTE: We assume a unit will not buy-out of a partial block to simplify calculations;
		/*Decide what unit will do for this block in RTD. A unit will either:
			Follow their DAM Schedule (ENERGY) [BLOCK_DAM_LBMP - BLOCK_COSTS]
			Buy-out of their DAM Schedule (BUYOUT) [BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES]
		*/
		/*For final profits, allocate parts to 'other' and 'physical', where 'other' represnts
			profits due to other transactions (e.g., buyouts) and 'physical' represents profits
			due to physically providing product. Later in the code, EFORd will only apply to physical.*/
	data blk_profits_dam_&tech.3;
		set blk_profits_dam_&tech.2;
		format RTD_DISPATCH $7.;
		if DAM_COMMIT = "ENERGY" then do;
		%macro toggle_buyout();
			RTD_CHECK_ENERGY   = BLOCK_DAM_LBMP - BLOCK_COSTS;
			RTD_CHECK_BUYOUT   = BLOCK_DAM_LBMP - BLOCK_RTD_LBMP - BUYOUT_FUEL_LOSSES;

			/*Potential profits if we allow for day-ahead buyout*/
			%if &enable_day_ahead_buyout. = yes %then %do;
				if (RTD_CHECK_ENERGY > RTD_CHECK_BUYOUT) then do;
					RTD_DISPATCH = "ENERGY";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
				end;
				else if (RTD_CHECK_BUYOUT >= RTD_CHECK_ENERGY) then do;
					RTD_DISPATCH = "BUYOUT";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_BUYOUT / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = RTD_CHECK_BUYOUT / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_PHYSICAL = 0;
				end;

			%end;
			/*If we don't allow for buyout behavior, simply set the dispatch to what was previously decided.*/
			%else %do;
					RTD_DISPATCH = "ENERGY";
					RTD_FINAL_HRLY_PROFIT = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
					RTD_FINAL_HRLY_OTHER = 0;
					RTD_FINAL_HRLY_PHYSICAL = RTD_CHECK_ENERGY / DAM_BLOCK_HOURS;
			%end;
		%mend;
		%toggle_buyout;

		label RTD_DISPATCH = 'Real-Time Dispatch (Energy/None)';
		label RTD_CHECK_ENERGY = 'Block Profits if Unit Dispatches Energy ($)';
		label RTD_CHECK_BUYOUT = 'Block Profits if Unit Buys Out of Day-Ahead Energy Commitment ($)';
		label RTD_FINAL_HRLY_PROFIT = 'Final Unit Profit in Each Hour Given Commitment Decision ($)';
		label RTD_FINAL_HRLY_OTHER = 'Other Component of Final Unit Profit in Each Hour Given Commitment Decision ($)';
		label RTD_FINAL_HRLY_PHYSICAL = 'Physical Component of Final Unit Profit in Each Hour Given Commitment Decision ($)';
		end;
	run;

	* 	 F: Merge block-level decision back on to hour-level data;
	proc sql;
		create table hr_profits_dam_&tech.2 as select
	    	a.*,
			b.DAM_COMMIT,
			b.RTD_DISPATCH,
			case when b.RTD_DISPATCH = "ENERGY" then a.ECONOMIC_FUEL else "" end as FIRED_ENERGY_FUEL label = 'Fuel Used in Energy Production (Gas/Oil)',
			b.RTD_FINAL_HRLY_PROFIT label = 'Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)',
			b.RTD_FINAL_HRLY_OTHER label = 'Other Component of Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)',
			b.RTD_FINAL_HRLY_PHYSICAL label = 'Physical Component of Final Unit Profit in Each Hour Given Commitment Decision ($/MWh)'
		from blk_hrly_profits_dam_&tech.2 as a
			left join blk_profits_dam_&tech.3 as b
			on a.AREA = b.AREA and a.DAM_BLOCK_NUMBER = b.DAM_BLOCK_NUMBER
		order by a.AREA, a.DATE, a.TIME;
	quit;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 6. DAM Non-Energy Hours - Flag No Commitment 		       									   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/* For all other DAM hours the unit is not committed*/
	data hr_profits_&tech.3;
		set hr_profits_dam_&tech.2;
		if missing(DAM_COMMIT) then DAM_COMMIT = "NONE";
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 7. RTD Energy Blocks - Determine if Profitable to Change Day-Ahead Non-Energy to 
							 Real-Time Energy Dispatch											   ;
	* NOTE: There are two ways a unit can decide to dispatch energy in real-time.
		1. By extending (in either direction) the length of a previous day-ahead commitment.
		2. A unit will start up for purely real-time energy dispatch if it can recover real-time
		   startup costs within 2 hours of starting.	;

	* NOTE: We solve this by evaluating potential real-time energy blocks. Blocks are formed by 
		 looking at hours where it would be profitable to commit if they had no day-ahead position. 
		 Hours already committed via day-ahead are included in these blocks because we need to 
		 determine if the unit has already budgeted for the unit to startup in a given block 
		 day-ahead.

	*    A. For each hour determine real-time energy profit left after buyout of day-ahead position;
	*    B. Flag potential real-time energy blocks;
	*    C. If any hour of the block was committed day-ahead, flag that startup cost has been paid ;
	*    D. For all other blocks, find first point (if any) where startup costs can be recovered 
			over a one (LMS/W) or two-hour (Frame) period. Flag all hours after this point for 
			energy dispatch																		   ;
	*    E. Update real time commitment to reflect flagged blocks ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	/*    A. For each hour determine real-time energy profit left after buyout of day-ahead position*/
	/*If from nothing, the RTD_ENERGY must be positive.*/

	data hr_profits_&tech.4 (drop = _:);
		set hr_profits_&tech.3;
		by AREA DATE TIME;
		retain _COUNTER _LAG1;

		if first.AREA then _COUNTER = 0;

		if DAM_COMMIT = "NONE" then RTD_POTENTIAL_ENERGY_PROFIT = RTD_HR_ENERGY_PROFIT;
		if RTD_DISPATCH = "ENERGY" then RTD_POTENTIAL_ENERGY_PROFIT = RTD_FINAL_HRLY_PROFIT;

	/*    B. Flag potential real-time energy blocks*/
		_LAG1 = lag1(RTD_POTENTIAL_ENERGY_PROFIT);
		if RTD_POTENTIAL_ENERGY_PROFIT > 0 and _LAG1 <= 0 then _COUNTER + 1;

		if RTD_POTENTIAL_ENERGY_PROFIT < 0 then RTD_POTENTIAL_ENERGY_PROFIT = .;
		else RTD_POTENTIAL_BLOCK_NUMBER = _COUNTER;

		label RTD_POTENTIAL_ENERGY_PROFIT = 'Real-Time Potential Run Block Energy Profit ($/MWh)';
		label RTD_POTENTIAL_BLOCK_NUMBER = 'Real-Time Potential Run Block Number';
	run;

	/*    C. If any hour of the block was committed day-ahead, flag that startup cost has been paid 
		(already accounted for in RTD_FINAL_HRLY_PROFIT)*/
	proc sql;
		create table hr_profits_&tech.5 as select
			*,
			max(case when RTD_DISPATCH = "ENERGY" then 1 else . end) as RTD_BLOCK_STARTUP_RECOVERED_DAM label = 'Flag if the RTD Energy Block is Already Assured Start-Up Recovery via DAM Commitment'
		from hr_profits_&tech.4
		group by AREA, RTD_POTENTIAL_BLOCK_NUMBER
		order by AREA, DATE, TIME;
	quit;

	/*    D. For all other blocks, find first point (if any) where startup costs can be recovered 
			over a one (LMS/W) or two-hour (Frame) period. Flag all hours after this point for 
			energy dispatch*/
	data hr_profits_&tech.6 (drop = _:);
		set hr_profits_&tech.5;
		by AREA DATE TIME;
		retain _LAG1 RTD_BLOCK_HOUR;

		_LAG1 = lag1(RTD_POTENTIAL_ENERGY_PROFIT);

		/*Label hours in each block*/
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) and missing(_LAG1) then RTD_BLOCK_HOUR = 0;
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) then RTD_BLOCK_HOUR + 1;
		else RTD_BLOCK_HOUR = .;

		/*Calculate Profit over Previous 2 Hours*/
		if ~missing(RTD_POTENTIAL_ENERGY_PROFIT) and RTD_BLOCK_STARTUP_RECOVERED_DAM ~= 1 then do;
			if missing(_LAG1) then TWO_HR_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
			else TWO_HR_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT + _LAG1;
		end;

		/*See if Startup Costs Recovered and flag first hour recovered*/
		/*We subtract 1 for the two-hour case because we're measuring at the second hour.*/
		if ("&tech." = "F" or "&tech." = "H" or "&tech." = "CCF" or "&tech." = "CCH") and TWO_HR_PROFIT > RTD_STARTUP_COST then RTD_START_HOUR_FLAG = max(RTD_BLOCK_HOUR - 1, 1);
		else if ("&tech." = "LMS" or "&tech." = "W") and RTD_POTENTIAL_ENERGY_PROFIT > RTD_STARTUP_COST and RTD_BLOCK_STARTUP_RECOVERED_DAM ~= 1 then RTD_START_HOUR_FLAG = RTD_BLOCK_HOUR;

		label RTD_BLOCK_HOUR = 'Count Block Hours in Sequence, Used for Real-Time Amortization Decision';
		label TWO_HR_PROFIT = 'Rolling Two-Hour Unit Profit, Used for Real-Time Amortization Decision';
		label RTD_START_HOUR_FLAG = 'Temporary Variable, Used to Flag When Startup Costs Are Recovered';
	run;

	/*Flag all hours after start is triggered for energy dispatch*/
	proc sql;
		create table hr_profits_&tech.7 (drop = RTD_START_HOUR_FLAG) as select
			*,
			min(RTD_START_HOUR_FLAG) as RTD_START_HOUR label = 'The Block Hour in Which Unit Can Start and Be Assured Amortized Cost Recovery'
		from hr_profits_&tech.6
		group by AREA, RTD_POTENTIAL_BLOCK_NUMBER
		order by AREA, DATE, TIME;
	quit;

	/*    E. Update real time commitment to reflect flagged blocks*/
	/*1. If extending a DAM block, no additional startup cost [RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and DAM_COMMIT ~= "ENERGY"]*/
	/*2. If starting up "fresh", needs additional startup cost */

	data hr_profits_&tech.8 (drop = _: /*RTD_BLOCK_STARTUP_RECOVERED_DAM*/ RTD_BLOCK_HOUR TWO_HR_PROFIT RTD_START_HOUR);
		set hr_profits_&tech.7;
		by AREA;
		retain _COUNTER _LAG1;
		if first.AREA then _COUNTER = 0;

		/*Flag Start Hour for Both Types of Real-Time Block (day-ahead extensions and new blocks)*/
		_LAG1 = lag1(RTD_BLOCK_STARTUP_RECOVERED_DAM);
		if (RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and _LAG1 = .) or (~missing(RTD_START_HOUR) and RTD_START_HOUR = RTD_BLOCK_HOUR) then _COUNTER + 1;

		/*1. If extending a DAM committed energy block, there is no additional startup cost to apply*/
		if RTD_BLOCK_STARTUP_RECOVERED_DAM = 1 and DAM_COMMIT ~= "ENERGY" then do;
			/*None -> Energy*/
			RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
			RTD_FINAL_HRLY_OTHER = 0;
			RTD_FINAL_HRLY_PHYSICAL = RTD_HR_ENERGY_PROFIT;

			RTD_DISPATCH = "ENERGY";
			FIRED_ENERGY_FUEL = ECONOMIC_FUEL;
		end;
		/*2. Otherwise, we need to apply a startup cost in the first hour*/
		else if (~missing(RTD_START_HOUR) and RTD_START_HOUR <= RTD_BLOCK_HOUR) then do;
			/*Apply startup in first hour*/
			if RTD_BLOCK_HOUR = RTD_START_HOUR then do;
				/*None -> Energy*/
				RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT - RTD_STARTUP_COST;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = RTD_HR_ENERGY_PROFIT - RTD_STARTUP_COST;
			end;
			/*All other hours*/
			else do;
				/*None -> Energy*/
				RTD_FINAL_HRLY_PROFIT = RTD_POTENTIAL_ENERGY_PROFIT;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = RTD_POTENTIAL_ENERGY_PROFIT;
			end;

			RTD_DISPATCH = "ENERGY";
			FIRED_ENERGY_FUEL = ECONOMIC_FUEL;
		end;

		/*Update Block Number*/
		if RTD_DISPATCH = "ENERGY" then RTD_BLOCK_NUMBER = _COUNTER;
		label RTD_BLOCK_NUMBER = 'Real-Time Run Block Number';
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 8. RTD Non-Energy Hours - Set to None 	  	   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

	data hr_profits_&tech.9;
		set hr_profits_&tech.8;

		/*Only act on hours not committed energy*/
		if missing(RTD_DISPATCH) then do;
			/*No DAM commitment*/
			if DAM_COMMIT = "NONE" then do;
				RTD_FINAL_HRLY_PROFIT = 0;
				RTD_FINAL_HRLY_OTHER = 0;
				RTD_FINAL_HRLY_PHYSICAL = 0;
				RTD_DISPATCH = "NONE";
			end;
		end;
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 9. Min Run Addition - Determine when CC Unit will Run at Min Load							   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	/*Create a flag for the "next row" DAM startup. This is used during the intra-block period to see if the
		next block will startup day-ahead or in real time. The appropriate startup cost can then be compared
		to the losses by running between profitable energy blocks.*/
	data start_flag_&tech.;
		rec_no=_n_+1;
		set hr_profits_&tech.9 end=last;
		if not last 
			then set hr_profits_&tech.9 (keep=RTD_BLOCK_STARTUP_RECOVERED_DAM rename=(RTD_BLOCK_STARTUP_RECOVERED_DAM=NEXT_ROW_STARTUP_DAM)) point=rec_no;
		else call missing(NEXT_ROW_STARTUP_DAM);
	run;

	/*Flag blocks between run hours*/
	data cc_minrun_&tech. (drop = INTER_BLOCK INTER_COUNT_LAG);
		set start_flag_&tech.;
		by AREA DATE TIME;
		retain INTER_BLOCK INTER_COUNT_LAG;
		
		if first.AREA then do;
			INTER_BLOCK = .;
			INTER_COUNT_LAG = .;
		end;

		if lag(RTD_DISPATCH) = "ENERGY" and RTD_DISPATCH ~= "ENERGY" then do;
			INTER_BLOCK = 1;
			if missing(INTER_COUNT_LAG) then INTER_COUNT_LAG = 1;
			else INTER_COUNT_LAG = INTER_COUNT_LAG + 1;
		end;

		if lag(RTD_DISPATCH) ~= "ENERGY" and RTD_DISPATCH = "ENERGY" then INTER_BLOCK = .;
		if ~missing(INTER_BLOCK) then INTER_BLOCK_NUMBER = INTER_COUNT_LAG; 
		label INTER_BLOCK_NUMBER = 'Min Load Run Block Number';
	run;

	/*Collapse blocks between run blocks*/
	proc sql;
		create table cc_minrun_&tech.2 as select distinct
			AREA, INTER_BLOCK_NUMBER,
			max(NEXT_ROW_STARTUP_DAM) as COMPARE_DAM label = 'Flag if Losses Should be Compared to DAM Start or RTD Start',
			count(INTER_BLOCK_NUMBER) as INTER_BLOCK_HOURS label = 'Hours in Inter-Block',
			sum(RTD_HR_ENERGY_PROFIT_MIN) as RTD_BLOCK_ENERGY_PROFIT label = 'Real-Time Inter-Block Energy Profit ($)',
			min(DAM_STARTUP_COST) as DAM_STARTUP_COST,
			min(RTD_STARTUP_COST) as RTD_STARTUP_COST,
			/*This is the amount of expenses saved. For now, allocate this savings to the inter-block period*/
			min(E_BUYOUT_STARTUP_FUEL_LOSSES) as DAM_STARTUP_LOSS,
			case when calculated COMPARE_DAM = 1 then 
				/*DAM gas has already been purchased, so the unit takes a loss on saving it in real-time (DAM_STARTUP_LOSS).*/
				calculated RTD_BLOCK_ENERGY_PROFIT + calculated DAM_STARTUP_COST - calculated DAM_STARTUP_LOSS else
				calculated RTD_BLOCK_ENERGY_PROFIT + calculated RTD_STARTUP_COST
					end as SAVED_STARTUP_EXPENSE
		from cc_minrun_&tech.
		where ~missing(INTER_BLOCK_NUMBER)
		group by AREA, INTER_BLOCK_NUMBER
		having INTER_BLOCK_HOURS <= 24
		order by AREA, INTER_BLOCK_NUMBER;
	quit;

	/*Merge on Offset*/
	proc sql;
		create table cc_minrun_&tech.3 as select
	    	a.*,
			b.SAVED_STARTUP_EXPENSE / b.INTER_BLOCK_HOURS as INTER_BLOCK_OFFSET label = '$/MWh Recovered by Running at Min Load instead of Restarting'
		from cc_minrun_&tech. as a
			left join cc_minrun_&tech.2 (where = (SAVED_STARTUP_EXPENSE > 0)) as b
			on a.AREA = b.AREA and a.INTER_BLOCK_NUMBER = b.INTER_BLOCK_NUMBER
		order by AREA, DATE, TIME;
	quit;

	/*Update Commitment/Dispatch Variables*/
	data cc_minrun_&tech.4;
		set cc_minrun_&tech.3;

		if ~missing(INTER_BLOCK_OFFSET) then do;
			RTD_DISPATCH = "ENERGY";
			RTD_FINAL_HRLY_PROFIT = INTER_BLOCK_OFFSET;
			RTD_FINAL_HRLY_OTHER = 0;
			RTD_FINAL_HRLY_PHYSICAL = INTER_BLOCK_OFFSET;
			FIRED_ENERGY_FUEL = ECONOMIC_FUEL;
		end;
	run;

	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	* 10. EFORd Adjustment  												   						   ;
	*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
	/*The EFORd adjustment impacts hours when the unit is committed to physically provide energy  
		services. It does not impact other profits due to buyouts, etc.*/

	data hr_profits_&tech.10;
		set cc_minrun_&tech.4;
		EFORd_PROFIT_LOSS = (RTD_FINAL_HRLY_PHYSICAL * (1-EFORd_&tech.));
		RTD_FINAL_HRLY_PROFIT = RTD_FINAL_HRLY_PROFIT - (RTD_FINAL_HRLY_PHYSICAL * (1-EFORd_&tech.));
		label EFORd_PROFIT_LOSS = 'Reduction in Profit due to Unit EFORd ($/MWh)';
	run;

	/*Save results*/
	data neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;
		set hr_profits_&tech.10;
		proc sort; by AREA DATE TIME;
	run;
%mend eas_profits;
%eas_profits(tech=CCF);
%eas_profits(tech=CCH);

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;
* 11. Summarize Model Results 	  	   														   ;
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*;

%macro output_diagnostics(tech=);
	/*Final Net EAS 3-Year Average*/
	data flag_starts_&tech.;
		set neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;
		if RTD_DISPATCH = "ENERGY" and missing(lag(FIRED_ENERGY_FUEL)) then UNIT_START = 1;
	run;

	proc sql;
		/*Sum to Monthly Level*/ 
		create table final_dispatch_mo_&tech. as select distinct
			AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, month(DATE) as MONTH,
			sum(RTD_FINAL_HRLY_PROFIT) as FINAL_HRLY_PROFIT_&tech.,
			sum(case when RTD_DISPATCH = "ENERGY" then 1 else 0 end) as FINAL_HRLY_COUNT_&tech.,
			sum(UNIT_START) as FINAL_START_COUNT_&tech.
		from flag_starts_&tech.
		group by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, MONTH
		order by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR, MONTH;

		/*Average Each Month (over 3-years)*/
		create table final_dispatch_mo_&tech.2 as select distinct
			AREA, ZONE, CAPACITY_ZONE, MONTH,
			mean(FINAL_HRLY_PROFIT_&tech.) as FINAL_HRLY_PROFIT_&tech.,
			mean(FINAL_HRLY_COUNT_&tech.) as FINAL_HRLY_COUNT_&tech.,
			mean(FINAL_START_COUNT_&tech.) as FINAL_START_COUNT_&tech.
		from final_dispatch_mo_&tech.
		group by AREA, ZONE, CAPACITY_ZONE, MONTH
		order by AREA, ZONE, CAPACITY_ZONE, MONTH;

		/*Sum to Final*/
		create table final_dispatch_mo_&tech.3 as select distinct
			catx("_",AREA,ZONE) as LOOKUP,
			AREA, ZONE, CAPACITY_ZONE,
			sum(FINAL_HRLY_PROFIT_&tech.)/1000 + (&vss_revenue. + &ancillary_revenue.) as FINAL_HRLY_PROFIT_&tech.,
			sum(FINAL_HRLY_COUNT_&tech.) as FINAL_HRLY_COUNT_&tech.,
			sum(FINAL_START_COUNT_&tech.) as FINAL_START_COUNT_&tech.
		from final_dispatch_mo_&tech.2
		group by AREA, ZONE, CAPACITY_ZONE
		order by AREA, ZONE, CAPACITY_ZONE;
	quit;

	/*Overview*/
	proc summary data = neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;
		class MODEL_YEAR AREA DAM_COMMIT RTD_DISPATCH;
		output out = summary_area_year_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_area_year_&tech.;
		format LOOKUP $100.;
		set summary_area_year_&tech.;
		LOOKUP = catx("-",MODEL_YEAR,AREA,DAM_COMMIT,RTD_DISPATCH);
	run;

	/*Overview - By Month*/
	data summary_area_month_&tech.;
		set neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;
		YEAR = year(DATE);
		MONTH = month(DATE);
	run;
	proc summary data = summary_area_month_&tech.;
		class YEAR MONTH AREA DAM_COMMIT RTD_DISPATCH;
		output out = summary_area_month_&tech.2 sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_area_month_&tech.;
		format LOOKUP $100.;
		set summary_area_month_&tech.2;
		LOOKUP = catx("-",YEAR,MONTH,AREA,DAM_COMMIT,RTD_DISPATCH);
	run;

	/*Oil/Gas*/
	proc summary data = neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE. (where = (RTD_DISPATCH = "ENERGY"));
		class MODEL_YEAR AREA FIRED_ENERGY_FUEL;
		output out = summary_oil_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;
	data lk_summary_oil_&tech.;
		format LOOKUP $100.;
		set summary_oil_&tech.;
		LOOKUP = catx("-",MODEL_YEAR,AREA,FIRED_ENERGY_FUEL);
	run;

	/*Summarize by Energy/Ancillary*/
	proc summary nway data = neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;
		class RTD_DISPATCH AREA MODEL_YEAR;
		output out = summ_&tech. sum = /autoname;
		vars RTD_FINAL_HRLY_PROFIT;
	run;

	/*Summarize by Year*/
	proc sql;
		create table yearly_summ_&tech. as select distinct
			catx("-",MODEL_YEAR,AREA) as LOOKUP,
			AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR,
			sum(RTD_FINAL_HRLY_PROFIT)/1000 + (&vss_revenue. + &ancillary_revenue.) as FINAL_HRLY_PROFIT_&tech.,
			sum(case when RTD_DISPATCH = "ENERGY" then 1 else 0 end) as FINAL_HRLY_COUNT_&tech.,
			count(distinct case when RTD_DISPATCH = "ENERGY" then RTD_BLOCK_NUMBER else . end) as FINAL_START_COUNT_&tech.,
			mean((DAM_LBMP * LOE) - DAM_GAS_COST) as SPARK_SPREAD_&tech.
		from neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.
		group by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR
		order by AREA, ZONE, CAPACITY_ZONE, MODEL_YEAR;
	quit;

/*	data hourly_data_&tech.;*/
/*		retain  DATE TIME Zone AREA LOE DAM_LBMP RTD_LBMP DAM_30MIN DAM_10MIN_NONSPIN RTD_30MIN RTD_10MIN_NONSPIN DAM_GAS_PRICE RTD_GAS_PRICE_BUY RTD_GAS_PRICE_SELL ULSD_PRICE DAM_GAS_COST RTD_GAS_COST OIL_COST ECONOMIC_FUEL E_BUYOUT_FUEL_LOSS E_BUYOUT_FUEL_LOSS_COST DAM_GAS_SPARK_SPREAD DAM_HR_ENERGY_PROFIT RTD_HR_ENERGY_PROFIT DAM_STARTUP_COST RTD_STARTUP_COST E_BUYOUT_STARTUP_FUEL_LOSSES DAM_POTENTIAL_ENERGY_PROFIT DAM_POTENTIAL_BLOCK_NUMBER DAM_BLOCK_ENERGY_PROFIT DAM_BLOCK_NUMBER RTD_POTENTIAL_ENERGY_PROFIT RTD_POTENTIAL_BLOCK_NUMBER RTD_BLOCK_NUMBER DAM_COMMIT RTD_DISPATCH DAM_ENERGY RTD_ENERGY RTD_FINAL_HRLY_OTHER RTD_FINAL_HRLY_PHYSICAL EFORd_PROFIT_LOSS RTD_FINAL_HRLY_PROFIT;*/
/*		set neteas.modelout_cc_&tech._&enable_gas_only._&enable_LOE.;*/
/*		keep DATE TIME Zone AREA LOE DAM_LBMP RTD_LBMP DAM_30MIN DAM_10MIN_NONSPIN RTD_30MIN RTD_10MIN_NONSPIN DAM_GAS_PRICE RTD_GAS_PRICE_BUY RTD_GAS_PRICE_SELL ULSD_PRICE DAM_GAS_COST RTD_GAS_COST OIL_COST ECONOMIC_FUEL E_BUYOUT_FUEL_LOSS E_BUYOUT_FUEL_LOSS_COST DAM_GAS_SPARK_SPREAD DAM_HR_ENERGY_PROFIT RTD_HR_ENERGY_PROFIT DAM_STARTUP_COST RTD_STARTUP_COST E_BUYOUT_STARTUP_FUEL_LOSSES DAM_POTENTIAL_ENERGY_PROFIT DAM_POTENTIAL_BLOCK_NUMBER DAM_BLOCK_ENERGY_PROFIT DAM_BLOCK_NUMBER RTD_POTENTIAL_ENERGY_PROFIT RTD_POTENTIAL_BLOCK_NUMBER RTD_BLOCK_NUMBER DAM_COMMIT RTD_DISPATCH DAM_ENERGY RTD_ENERGY RTD_FINAL_HRLY_OTHER RTD_FINAL_HRLY_PHYSICAL EFORd_PROFIT_LOSS RTD_FINAL_HRLY_PROFIT;*/
/**/
/*		if DAM_COMMIT = "ENERGY" then DAM_ENERGY = 1;*/
/*		if RTD_DISPATCH = "ENERGY" then RTD_ENERGY = 1;*/
/*		DAM_GAS_SPARK_SPREAD = (DAM_LBMP * LOE) - DAM_GAS_COST;*/
/*	run;*/;

	%macro export(data =, sheet =, file =);
		proc export data = &data. outfile = "&outpath.\&file..xlsx"
			dbms = XLSX replace;
			sheet = "&sheet.";
		run;
	%mend;
	
	%export(data=final_dispatch_mo_&tech.3, 	sheet=Final EAS &tech., 	file=&output_name.);
	%export(data=lk_summary_area_year_&tech., 	sheet=Results &tech., 		file=&output_name.);
	%export(data=lk_summary_area_month_&tech., 	sheet=Results M &tech., 		file=&output_name.);
	%export(data=lk_summary_oil_&tech., 		sheet=Oil Burn &tech., 		file=&output_name.);
	%export(data=summ_&tech., 					sheet=EAS Rev &tech., 		file=&output_name.);
	%export(data=yearly_summ_&tech., 			sheet=EAS Yr &tech., 		file=&output_name.);
/*	%export(data=hourly_data_&tech., 			sheet=&tech. Hourly Data, 	file=&output_name. Hourly);*/
%mend;
%output_diagnostics(tech=CCF);
%output_diagnostics(tech=CCH);
